@@ -32,6 +32,7 @@
 #include <inttypes.h>
 #include <string.h>
 #include <ctype.h>
+#include <wctype.h>
 
 #include <unistd.h>
 #include <errno.h>
@@ -78,9 +79,14 @@ void tc_print_buf(char *buf, size_t size) {
 }
 
 void tc_printf(char *fmt, ...) {
-	char *buf;
 	va_list args;
 	va_start(args, fmt);
+	if (tc.tee_fd == -1) {
+		vprintf(fmt, args);
+		va_end(args);
+		return;
+	}
+	char *buf;
 	int size = vasprintf(&buf, fmt, args);
 	va_end(args);
 	if (size >= 0) {
@@ -89,25 +95,97 @@ void tc_printf(char *fmt, ...) {
 	}
 }
 
+static int tc_str_valid(char *data, uint32_t size) {
+	int length;
+	wchar_t dest;
+
+	mbtowc(NULL, NULL, 0);
+	while ((length = mbtowc(&dest, data, size)) > -1 && size > 0) {
+		if (length == 0)
+			++length;
+		data += length;
+		size -= length;
+	}
+	if (length == -1)
+		return 0;
+	return 1;
+}
+
+static void tc_print_str(char *data, uint32_t size) {
+	if (tc_str_valid(data, size)) {
+		wchar_t dest;
+		int length;
+		mbtowc (NULL, NULL, 0);
+		while ((length = mbtowc(&dest, data, size)) > -1 && size > 0) {
+			if (dest >= 0x20) {
+				tc_printf ("%lc", dest);
+			}
+			else {
+				switch (dest) {
+				case 0x00:
+					tc_printf("\\0");
+					break;
+				case 0x07:
+					tc_printf("\\a");
+					break;
+				case 0x08:
+					tc_printf("\\b");
+					break;
+				case 0x09:
+					tc_printf("\\t");
+					break;
+				case 0x0A:
+					tc_printf("\\n");
+					break;
+				case 0x0B:
+					tc_printf("\\v");
+					break;
+				case 0x0C:
+					tc_printf("\\f");
+					break;
+				case 0x0D:
+					tc_printf("\\r");
+					break;
+				default:
+					tc_printf("\\x%02lX", 
+						(unsigned long int)dest);
+					break;
+				}
+				if (length == 0) 
+					++length;
+			}
+			size -= length;
+			data += length;
+		}
+	}
+	else {
+		while (size-- > 0) {
+			tc_printf("\\x%02X", (unsigned char)*data);
+			data++;
+		}
+	}
+}
+
 static void tc_print_fields(struct tnt_tuple *tu) {
 	struct tnt_iter ifl;
 	tnt_iter(&ifl, tu);
 	while (tnt_next(&ifl)) {
 		if (TNT_IFIELD_IDX(&ifl) != 0)
 			tc_printf(", ");
+		tc_printf("'");
 		char *data = TNT_IFIELD_DATA(&ifl);
 		uint32_t size = TNT_IFIELD_SIZE(&ifl);
-		if (!isprint(data[0]) && (size == 4 || size == 8)) {
-			if (size == 4) {
-				uint32_t i = *((uint32_t*)data);
-				tc_printf("%"PRIu32, i);
-			} else {
-				uint64_t i = *((uint64_t*)data);
-				tc_printf("%"PRIu64, i);
-			}
-		} else {
-			tc_printf("'%-.*s'", size, data);
+		switch (size) {
+		case 4:
+			tc_printf("%"PRIu32, *((uint32_t*)data));
+			break;
+		case 8:
+			tc_printf("%"PRIu64, *((uint64_t*)data));
+			break;
+		default:
+			tc_print_str(data, size);
 		}
+		tc_printf("'");
 	}
 	if (ifl.status == TNT_ITER_FAIL)
 		tc_printf("<parsing error>");
@@ -154,11 +232,24 @@ tc_printer_tarantool(struct tnt_log_header_v11 *hdr,
 	}
 }
 
+static void
+tc_printer_raw(struct tnt_log_header_v11 *hdr, struct tnt_request *r)
+{
+	if (tc.opt.raw_with_headers) {
+		fwrite(&tnt_log_marker_v11,
+		       sizeof(tnt_log_marker_v11), 1, stdout);
+	}
+	fwrite(hdr, sizeof(*hdr), 1, stdout);
+	fwrite(r->origin, r->origin_size, 1, stdout);
+}
+
 tc_printerf_t tc_print_getcb(const char *name)
 {
 	if (name == NULL)
 		return tc_printer_tarantool;
 	if (!strcasecmp(name, "tarantool"))
 		return tc_printer_tarantool;
+	if (!strcasecmp(name, "raw"))
+		return tc_printer_raw;
 	return NULL;
 }
