@@ -1,5 +1,8 @@
 #!/bin/sh -x
 
+# A wrapper script to run a single tarantool instance
+# and restart it when it crashes
+
 export PATH=$PATH:/usr/local/bin
 
 NAME="tarantool_box"
@@ -8,12 +11,17 @@ INST=$(basename $0 .sh)
 CONF="/usr/local/etc/${INST}.cfg"
 LOGDIR="/var/${INST}/logs"
 WRAP_PIDFILE="/var/${INST}/wrapper.pid"
+# set to get restart emails
+#MAILTO=""
 
 exec <&-
 
 report()
 {
-        tail -n 500 ${LOGDIR}/tarantool.log | mail -s "${@}" ${MAILTO}
+        if [ "${MAILTO}" ]; then
+                tail -n 500 ${LOGDIR}/tarantool.log | mail ${MAILTO} -s "\"${INST} is restarted\""
+        fi
+        echo \""${@}"\" >> ${LOGDIR}/wrapper.log
 }
 
 runtarantool()
@@ -22,18 +30,17 @@ runtarantool()
         ${BINARY} ${OPTIONS} --config ${CONF} 2>&1 </dev/null &
         wait
         RC=${?}
-        report "${INST} restarted! "`date '+%Y-%m-%d %H:%M:%S'`" exit code $RC"
-        echo "${INST} restarted! "`date '+%Y-%m-%d %H:%M:%S'`" exit code $RC<br>" >> /var/tmp/error.txt
+        report "${INST}: ${BINARY} ${OPTIONS} --config ${CONF} died prematurely "`date '+%Y-%m-%d %H:%M:%S'`" exit code $RC"
         sleep 2
 }
 
 {
-        ulimit -Hc unlimited
+        ulimit -c unlimited
         runtarantool
 
         while true
         do
-                ulimit -Hc 0
+                ulimit -c 0
                 runtarantool
         done
 } &
