@@ -44,9 +44,8 @@ struct mh_i32ptr_node_t {
 };
 
 #define mh_node_t struct mh_i32ptr_node_t
-#define mh_hash_arg_t void *
+#define mh_arg_t void *
 #define mh_hash(a, arg) (a->key)
-#define mh_eq_arg_t void *
 #define mh_eq(a, b, arg) ((a->key) == (b->key))
 #include <mhash.h>
 
@@ -62,9 +61,8 @@ struct mh_i64ptr_node_t {
 
 #define mh_node_t struct mh_i64ptr_node_t
 #define mh_int_t u32
-#define mh_hash_arg_t void *
+#define mh_arg_t void *
 #define mh_hash(a, arg) ((u32)((a->key)>>33^(a->key)^(a->key)<<11))
-#define mh_eq_arg_t void *
 #define mh_eq(a, b, arg) ((a->key) == (b->key))
 #include <mhash.h>
 
@@ -82,7 +80,7 @@ static inline int lstrcmp(const void *a, const void *b)
 		return bl - al;
 	return memcmp(a, b, al);
 }
-#include <third_party/murmur_hash2.c>
+#include <third_party/PMurHash.h>
 #define mh_name _lstrptr
 struct mh_lstrptr_node_t {
 	const void *key;
@@ -91,15 +89,14 @@ struct mh_lstrptr_node_t {
 
 #define mh_node_t struct mh_lstrptr_node_t
 #define mh_int_t u32
-#define mh_hash_arg_t void *
+#define mh_arg_t void *
 static inline u32
-mh_strptr_hash(const mh_node_t *a, mh_hash_arg_t arg) {
+mh_strptr_hash(const mh_node_t *a, mh_arg_t arg) {
 	(void) arg;
 	const void *_k = (a->key);
 	const u32 l = load_varint32(&_k);
-	return (u32) MurmurHash2(_k, l, 13);
+	return PMurHash32(13, _k, l);
 }
 #define mh_hash(a, arg) mh_strptr_hash(a, arg)
-#define mh_eq_arg_t void *
 #define mh_eq(a, b, arg) (lstrcmp(a->key, b->key) == 0)
 #include <mhash.h>
