@@ -51,6 +51,7 @@
 #include "lauxlib.h"
 #include "lualib.h"
 #include "box/box.h"
+#include "session.h"
 
 static const char *help =
 	"available commands:" CRLF
@@ -72,7 +73,7 @@ static const char *help =
 static const char *unknown_command = "unknown command. try typing help." CRLF;
 
 
-#line 76 "src/admin.m"
+#line 77 "src/admin.m"
 static const int admin_start = 1;
 static const int admin_first_final = 135;
 static const int admin_error = 0;
@@ -80,7 +81,7 @@ static const int admin_error = 0;
 static const int admin_en_main = 1;
 
 
-#line 75 "src/admin.rl"
+#line 76 "src/admin.rl"
 
 
 struct salloc_stat_admin_cb_ctx {
@@ -89,7 +90,7 @@ struct salloc_stat_admin_cb_ctx {
 };
 
 static int
-salloc_stat_admin_cb(const struct slab_class_stats *cstat, void *cb_ctx)
+salloc_stat_admin_cb(const struct slab_cache_stats *cstat, void *cb_ctx)
 {
 	struct salloc_stat_admin_cb_ctx *ctx = cb_ctx;
 
@@ -201,8 +202,8 @@ static int
 admin_dispatch(struct ev_io *coio, struct iobuf *iobuf, lua_State *L)
 {
 	struct ibuf *in = &iobuf->in;
-	struct tbuf *out = tbuf_alloc(fiber->gc_pool);
-	struct tbuf *err = tbuf_alloc(fiber->gc_pool);
+	struct tbuf *out = tbuf_new(fiber->gc_pool);
+	struct tbuf *err = tbuf_new(fiber->gc_pool);
 	int cs;
 	char *p, *pe;
 	char *strstart, *strend;
@@ -217,12 +218,12 @@ admin_dispatch(struct ev_io *coio, struct iobuf *iobuf, lua_State *L)
 	p = in->pos;
 
 	
-#line 221 "src/admin.m"
+#line 222 "src/admin.m"
 	{
 	cs = admin_start;
 	}
 
-#line 226 "src/admin.m"
+#line 227 "src/admin.m"
 	{
 	if ( p == pe )
 		goto _test_eof;
@@ -285,15 +286,15 @@ case 6:
 	}
 	goto st0;
 tr13:
-#line 308 "src/admin.rl"
+#line 309 "src/admin.rl"
 	{slab_validate(); ok(out);}
 	goto st135;
 tr20:
-#line 296 "src/admin.rl"
+#line 297 "src/admin.rl"
 	{return -1;}
 	goto st135;
 tr25:
-#line 223 "src/admin.rl"
+#line 224 "src/admin.rl"
 	{
 			start(out);
 			tbuf_append(out, help, strlen(help));
@@ -301,9 +302,9 @@ tr25:
 		}
 	goto st135;
 tr36:
-#line 282 "src/admin.rl"
+#line 283 "src/admin.rl"
 	{strend = p;}
-#line 229 "src/admin.rl"
+#line 230 "src/admin.rl"
 	{
 			strstart[strend-strstart]='\0';
 			start(out);
@@ -312,7 +313,7 @@ tr36:
 		}
 	goto st135;
 tr43:
-#line 236 "src/admin.rl"
+#line 237 "src/admin.rl"
 	{
 			if (reload_cfg(err))
 				fail(out, err);
@@ -321,13 +322,13 @@ tr43:
 		}
 	goto st135;
 tr67:
-#line 306 "src/admin.rl"
+#line 307 "src/admin.rl"
 	{coredump(60); ok(out);}
 	goto st135;
 tr76:
-#line 243 "src/admin.rl"
+#line 244 "src/admin.rl"
 	{
-			int ret = snapshot(NULL, 0);
+			int ret = snapshot();
 
 			if (ret == 0)
 				ok(out);
@@ -340,9 +341,9 @@ tr76:
 		}
 	goto st135;
 tr98:
-#line 292 "src/admin.rl"
+#line 293 "src/admin.rl"
 	{ state = false; }
-#line 256 "src/admin.rl"
+#line 257 "src/admin.rl"
 	{
 			strstart[strend-strstart] = '\0';
 			if (errinj_set_byname(strstart, state)) {
@@ -354,9 +355,9 @@ tr98:
 		}
 	goto st135;
 tr101:
-#line 291 "src/admin.rl"
+#line 292 "src/admin.rl"
 	{ state = true; }
-#line 256 "src/admin.rl"
+#line 257 "src/admin.rl"
 	{
 			strstart[strend-strstart] = '\0';
 			if (errinj_set_byname(strstart, state)) {
@@ -368,7 +369,7 @@ tr101:
 		}
 	goto st135;
 tr117:
-#line 211 "src/admin.rl"
+#line 212 "src/admin.rl"
 	{
 			start(out);
 			show_cfg(out);
@@ -376,15 +377,15 @@ tr117:
 		}
 	goto st135;
 tr131:
-#line 299 "src/admin.rl"
+#line 300 "src/admin.rl"
 	{start(out); fiber_info(out); end(out);}
 	goto st135;
 tr137:
-#line 298 "src/admin.rl"
+#line 299 "src/admin.rl"
 	{start(out); tarantool_info(out); end(out);}
 	goto st135;
 tr146:
-#line 217 "src/admin.rl"
+#line 218 "src/admin.rl"
 	{
 			start(out);
 			errinj_info(out);
@@ -392,33 +393,33 @@ tr146:
 		}
 	goto st135;
 tr152:
-#line 302 "src/admin.rl"
+#line 303 "src/admin.rl"
 	{start(out); palloc_stat(out); end(out);}
 	goto st135;
 tr160:
-#line 301 "src/admin.rl"
+#line 302 "src/admin.rl"
 	{start(out); show_slab(out); end(out);}
 	goto st135;
 tr164:
-#line 303 "src/admin.rl"
+#line 304 "src/admin.rl"
 	{start(out); show_stat(out);end(out);}
 	goto st135;
 st135:
 	if ( ++p == pe )
 		goto _test_eof135;
 case 135:
-#line 411 "src/admin.m"
+#line 412 "src/admin.m"
 	goto st0;
 tr14:
-#line 308 "src/admin.rl"
+#line 309 "src/admin.rl"
 	{slab_validate(); ok(out);}
 	goto st7;
 tr21:
-#line 296 "src/admin.rl"
+#line 297 "src/admin.rl"
 	{return -1;}
 	goto st7;
 tr26:
-#line 223 "src/admin.rl"
+#line 224 "src/admin.rl"
 	{
 			start(out);
 			tbuf_append(out, help, strlen(help));
@@ -426,9 +427,9 @@ tr26:
 		}
 	goto st7;
 tr37:
-#line 282 "src/admin.rl"
+#line 283 "src/admin.rl"
 	{strend = p;}
-#line 229 "src/admin.rl"
+#line 230 "src/admin.rl"
 	{
 			strstart[strend-strstart]='\0';
 			start(out);
@@ -437,7 +438,7 @@ tr37:
 		}
 	goto st7;
 tr44:
-#line 236 "src/admin.rl"
+#line 237 "src/admin.rl"
 	{
 			if (reload_cfg(err))
 				fail(out, err);
@@ -446,13 +447,13 @@ tr44:
 		}
 	goto st7;
 tr68:
-#line 306 "src/admin.rl"
+#line 307 "src/admin.rl"
 	{coredump(60); ok(out);}
 	goto st7;
 tr77:
-#line 243 "src/admin.rl"
+#line 244 "src/admin.rl"
 	{
-			int ret = snapshot(NULL, 0);
+			int ret = snapshot();
 
 			if (ret == 0)
 				ok(out);
@@ -465,9 +466,9 @@ tr77:
 		}
 	goto st7;
 tr99:
-#line 292 "src/admin.rl"
+#line 293 "src/admin.rl"
 	{ state = false; }
-#line 256 "src/admin.rl"
+#line 257 "src/admin.rl"
 	{
 			strstart[strend-strstart] = '\0';
 			if (errinj_set_byname(strstart, state)) {
@@ -479,9 +480,9 @@ tr99:
 		}
 	goto st7;
 tr102:
-#line 291 "src/admin.rl"
+#line 292 "src/admin.rl"
 	{ state = true; }
-#line 256 "src/admin.rl"
+#line 257 "src/admin.rl"
 	{
 			strstart[strend-strstart] = '\0';
 			if (errinj_set_byname(strstart, state)) {
@@ -493,7 +494,7 @@ tr102:
 		}
 	goto st7;
 tr118:
-#line 211 "src/admin.rl"
+#line 212 "src/admin.rl"
 	{
 			start(out);
 			show_cfg(out);
@@ -501,15 +502,15 @@ tr118:
 		}
 	goto st7;
 tr132:
-#line 299 "src/admin.rl"
+#line 300 "src/admin.rl"
 	{start(out); fiber_info(out); end(out);}
 	goto st7;
 tr138:
-#line 298 "src/admin.rl"
+#line 299 "src/admin.rl"
 	{start(out); tarantool_info(out); end(out);}
 	goto st7;
 tr147:
-#line 217 "src/admin.rl"
+#line 218 "src/admin.rl"
 	{
 			start(out);
 			errinj_info(out);
@@ -517,22 +518,22 @@ tr147:
 		}
 	goto st7;
 tr153:
-#line 302 "src/admin.rl"
+#line 303 "src/admin.rl"
 	{start(out); palloc_stat(out); end(out);}
 	goto st7;
 tr161:
-#line 301 "src/admin.rl"
+#line 302 "src/admin.rl"
 	{start(out); show_slab(out); end(out);}
 	goto st7;
 tr165:
-#line 303 "src/admin.rl"
+#line 304 "src/admin.rl"
 	{start(out); show_stat(out);end(out);}
 	goto st7;
 st7:
 	if ( ++p == pe )
 		goto _test_eof7;
 case 7:
-#line 536 "src/admin.m"
+#line 537 "src/admin.m"
 	if ( (*p) == 10 )
 		goto st135;
 	goto st0;
@@ -685,28 +686,28 @@ case 23:
 	}
 	goto tr33;
 tr33:
-#line 282 "src/admin.rl"
+#line 283 "src/admin.rl"
 	{strstart = p;}
 	goto st24;
 st24:
 	if ( ++p == pe )
 		goto _test_eof24;
 case 24:
-#line 696 "src/admin.m"
+#line 697 "src/admin.m"
 	switch( (*p) ) {
 		case 10: goto tr36;
 		case 13: goto tr37;
 	}
 	goto st24;
 tr34:
-#line 282 "src/admin.rl"
+#line 283 "src/admin.rl"
 	{strstart = p;}
 	goto st25;
 st25:
 	if ( ++p == pe )
 		goto _test_eof25;
 case 25:
-#line 710 "src/admin.m"
+#line 711 "src/admin.m"
 	switch( (*p) ) {
 		case 10: goto tr36;
 		case 13: goto tr37;
@@ -1156,28 +1157,28 @@ case 73:
 		goto tr91;
 	goto st0;
 tr91:
-#line 290 "src/admin.rl"
+#line 291 "src/admin.rl"
 	{ strstart = p; }
 	goto st74;
 st74:
 	if ( ++p == pe )
 		goto _test_eof74;
 case 74:
-#line 1167 "src/admin.m"
+#line 1168 "src/admin.m"
 	if ( (*p) == 32 )
 		goto tr92;
 	if ( 33 <= (*p) && (*p) <= 126 )
 		goto st74;
 	goto st0;
 tr92:
-#line 290 "src/admin.rl"
+#line 291 "src/admin.rl"
 	{ strend = p; }
 	goto st75;
 st75:
 	if ( ++p == pe )
 		goto _test_eof75;
 case 75:
-#line 1181 "src/admin.m"
+#line 1182 "src/admin.m"
 	switch( (*p) ) {
 		case 32: goto st75;
 		case 111: goto st76;
@@ -1869,7 +1870,7 @@ case 134:
 	_out: {}
 	}
 
-#line 314 "src/admin.rl"
+#line 315 "src/admin.rl"
 
 
 	in->pos = pe;
@@ -1889,10 +1890,16 @@ admin_handler(va_list ap)
 {
 	struct ev_io coio = va_arg(ap, struct ev_io);
 	struct iobuf *iobuf = va_arg(ap, struct iobuf *);
-	fiber_set_sid(fiber, box_sid());
 	lua_State *L = lua_newthread(tarantool_L);
 	int coro_ref = luaL_ref(tarantool_L, LUA_REGISTRYINDEX);
 	@try {
+		/*
+		 * Admin and iproto connections must have a
+		 * session object, representing the state of
+		 * a remote client: it's used in Lua
+		 * stored procedures.
+		 */
+		session_create(coio.fd);
 		for (;;) {
 			if (admin_dispatch(&coio, iobuf, L) < 0)
 				return;
@@ -1902,7 +1909,8 @@ admin_handler(va_list ap)
 	} @finally {
 		luaL_unref(tarantool_L, LUA_REGISTRYINDEX, coro_ref);
 		evio_close(&coio);
-		iobuf_destroy(iobuf);
+		iobuf_delete(iobuf);
+		session_destroy(fiber->sid);
 	}
 }
 
