@@ -29,7 +29,7 @@
 #include "box_lua.h"
 #include "lua/init.h"
 #include <fiber.h>
-#include "box.h"
+#include "box/box.h"
 #include "request.h"
 #include "txn.h"
 
@@ -45,6 +45,7 @@
 #include "tuple.h"
 #include "space.h"
 #include "port.h"
+#include "tbuf.h"
 
 /* contents of box.lua */
 extern const char box_lua[];
@@ -79,6 +80,9 @@ static const char *tuplelib_name = "box.tuple";
 static void
 lbox_pushtuple(struct lua_State *L, struct tuple *tuple);
 
+static struct tuple *
+lua_totuple(struct lua_State *L, int index);
+
 static inline struct tuple *
 lua_checktuple(struct lua_State *L, int narg)
 {
@@ -100,6 +104,17 @@ lua_istuple(struct lua_State *L, int narg)
 	return tuple;
 }
 
+static int
+lbox_tuple_new(lua_State *L)
+{
+	int argc = lua_gettop(L);
+	if (argc < 1)
+		luaL_error(L, "tuple.new(): bad arguments");
+	struct tuple *tuple = lua_totuple(L, 1);
+	lbox_pushtuple(L, tuple);
+	return 1;
+}
+
 static int
 lbox_tuple_gc(struct lua_State *L)
 {
@@ -145,14 +160,14 @@ lbox_tuple_slice(struct lua_State *L)
 	if (end <= start)
 		luaL_error(L, "tuple.slice(): start must be less than end");
 
-	u8 *field = tuple->data;
-	int fieldno = 0;
-	int stop = end - 1;
+	const void *field = tuple->data;
+	u32 fieldno = 0;
+	u32 stop = end - 1;
 
-	while (field < tuple->data + tuple->bsize) {
-		size_t len = load_varint32((void **) &field);
+	while (field < (const void *) tuple->data + tuple->bsize) {
+		size_t len = load_varint32(&field);
 		if (fieldno >= start) {
-			lua_pushlstring(L, (char *) field, len);
+			lua_pushlstring(L, field, len);
 			if (fieldno == stop)
 				break;
 		}
@@ -183,8 +198,8 @@ transform_calculate(struct lua_State *L, struct tuple *tuple,
 		    size_t lr[2])
 {
 	/* calculate size of the new tuple */
-	void *tuple_end = tuple->data + tuple->bsize;
-	void *tuple_field = tuple->data;
+	const void *tuple_end = tuple->data + tuple->bsize;
+	const void *tuple_field = tuple->data;
 
 	lr[0] = tuple_range_size(&tuple_field, tuple_end, offset);
 
@@ -219,14 +234,6 @@ transform_calculate(struct lua_State *L, struct tuple *tuple,
 	return lr[0] + mid + lr[1];
 }
 
-static inline void
-transform_set_field(u8 **ptr, const void *data, size_t size)
-{
-	*ptr = save_varint32(*ptr, size);
-	memcpy(*ptr, data, size);
-	*ptr += size;
-}
-
 /**
  * Tuple transforming function.
  *
@@ -270,23 +277,23 @@ lbox_tuple_transform(struct lua_State *L)
 
 	/* construct tuple */
 	memcpy(dest->data, tuple->data, lr[0]);
-	u8 *ptr = dest->data + lr[0];
+	void *ptr = dest->data + lr[0];
 	for (int i = 4; i <= argc; i++) {
 		switch (lua_type(L, i)) {
 		case LUA_TNUMBER: {
 			u32 v = lua_tonumber(L, i);
-			transform_set_field(&ptr, &v, sizeof(v));
+			ptr = pack_lstr(ptr, &v, sizeof(v));
 			break;
 		}
 		case LUA_TCDATA: {
 			u64 v = tarantool_lua_tointeger64(L, i);
-			transform_set_field(&ptr, &v, sizeof(v));
+			ptr = pack_lstr(ptr, &v, sizeof(v));
 			break;
 		}
 		case LUA_TSTRING: {
 			size_t field_size = 0;
 			const char *v = luaL_checklstring(L, i, &field_size);
-			transform_set_field(&ptr, v, field_size);
+			ptr = pack_lstr(ptr, v, field_size);
 			break;
 		}
 		default:
@@ -319,9 +326,9 @@ tuple_find(struct lua_State *L, struct tuple *tuple, size_t offset,
 	int idx = offset;
 	if (idx >= tuple->field_count)
 		return 0;
-	u8 *field = tuple_field(tuple, idx);
-	while (field < tuple->data + tuple->bsize) {
-		size_t len = load_varint32((void **) &field);
+	const void *field = tuple_field(tuple, idx);
+	while (field < (const void *) tuple->data + tuple->bsize) {
+		size_t len = load_varint32(&field);
 		if (len == key_size && (memcmp(field, key, len) == 0)) {
 			lua_pushinteger(L, idx);
 			if (!all)
@@ -387,17 +394,34 @@ static int
 lbox_tuple_unpack(struct lua_State *L)
 {
 	struct tuple *tuple = lua_checktuple(L, 1);
-	u8 *field = tuple->data;
+	const void *field = tuple->data;
 
-	while (field < tuple->data + tuple->bsize) {
-		size_t len = load_varint32((void **) &field);
-		lua_pushlstring(L, (char *) field, len);
+	while (field < (const void *) tuple->data + tuple->bsize) {
+		size_t len = load_varint32(&field);
+		lua_pushlstring(L, field, len);
 		field += len;
 	}
 	assert(lua_gettop(L) == tuple->field_count + 1);
 	return tuple->field_count;
 }
 
+static int
+lbox_tuple_totable(struct lua_State *L)
+{
+	struct tuple *tuple = lua_checktuple(L, 1);
+	lua_newtable(L);
+	int index = 1;
+	const void *field = tuple->data;
+	while (field < (const void *) tuple->data + tuple->bsize) {
+		size_t len = load_varint32(&field);
+		lua_pushnumber(L, index++);
+		lua_pushlstring(L, field, len);
+		lua_rawset(L, -3);
+		field += len;
+	}
+	return 1;
+}
+
 /**
  * Implementation of tuple __index metamethod.
  *
@@ -415,14 +439,15 @@ lbox_tuple_index(struct lua_State *L)
 		if (i >= tuple->field_count)
 			luaL_error(L, "%s: index %d is out of bounds (0..%d)",
 				   tuplelib_name, i, tuple->field_count-1);
-		void *field = tuple_field(tuple, i);
+		const void *field = tuple_field(tuple, i);
 		u32 len = load_varint32(&field);
 		lua_pushlstring(L, field, len);
 		return 1;
 	}
 	/* If we got a string, try to find a method for it. */
+	const char *sz = luaL_checkstring(L, 2);
 	lua_getmetatable(L, 1);
-	lua_getfield(L, -1, lua_tostring(L, 2));
+	lua_getfield(L, -1, sz);
 	return 1;
 }
 
@@ -431,7 +456,7 @@ lbox_tuple_tostring(struct lua_State *L)
 {
 	struct tuple *tuple = lua_checktuple(L, 1);
 	/* @todo: print the tuple */
-	struct tbuf *tbuf = tbuf_alloc(fiber->gc_pool);
+	struct tbuf *tbuf = tbuf_new(fiber->gc_pool);
 	tuple_print(tbuf, tuple->field_count, tuple->data);
 	lua_pushlstring(L, tbuf->data, tbuf->size);
 	return 1;
@@ -461,7 +486,7 @@ lbox_tuple_next(struct lua_State *L)
 {
 	struct tuple *tuple = lua_checktuple(L, 1);
 	int argc = lua_gettop(L) - 1;
-	u8 *field = NULL;
+	const void *field = NULL;
 	size_t len;
 
 	if (argc == 0 || (argc == 1 && lua_type(L, 2) == LUA_TNIL))
@@ -472,11 +497,11 @@ lbox_tuple_next(struct lua_State *L)
 		luaL_error(L, "tuple.next(): bad arguments");
 
 	(void)field;
-	assert(field >= tuple->data);
-	if (field < tuple->data + tuple->bsize) {
-		len = load_varint32((void **) &field);
-		lua_pushlightuserdata(L, field + len);
-		lua_pushlstring(L, (char *) field, len);
+	assert(field >= (const void *) tuple->data);
+	if (field < (const void *) tuple->data + tuple->bsize) {
+		len = load_varint32(&field);
+		lua_pushlightuserdata(L, (void *) field + len);
+		lua_pushlstring(L, field, len);
 		return 2;
 	}
 	lua_pushnil(L);
@@ -495,7 +520,7 @@ lbox_tuple_pairs(struct lua_State *L)
 	return 3;
 }
 
-static const struct luaL_reg lbox_tuple_meta [] = {
+static const struct luaL_reg lbox_tuple_meta[] = {
 	{"__gc", lbox_tuple_gc},
 	{"__len", lbox_tuple_len},
 	{"__index", lbox_tuple_index},
@@ -507,6 +532,12 @@ static const struct luaL_reg lbox_tuple_meta [] = {
 	{"find", lbox_tuple_find},
 	{"findall", lbox_tuple_findall},
 	{"unpack", lbox_tuple_unpack},
+	{"totable", lbox_tuple_totable},
+	{NULL, NULL}
+};
+
+static const struct luaL_reg lbox_tuplelib[] = {
+	{"new", lbox_tuple_new},
 	{NULL, NULL}
 };
 
@@ -527,12 +558,22 @@ lbox_checkiterator(struct lua_State *L, int i)
 }
 
 static void
-lbox_pushiterator(struct lua_State *L, struct iterator *it)
+lbox_pushiterator(struct lua_State *L, Index *index,
+                  struct iterator *it, enum iterator_type type,
+                  const void *key, size_t size, int part_count)
 {
-	void **ptr = lua_newuserdata(L, sizeof(void *));
+	struct {
+		struct iterator *it;
+		char key[];
+	} *holder = lua_newuserdata(L, sizeof(void *) + size);
 	luaL_getmetatable(L, iteratorlib_name);
 	lua_setmetatable(L, -2);
-	*ptr = it;
+
+	holder->it = it;
+	memcpy(holder->key, key, size);
+
+	[index initIterator: it :type :(key ? holder->key : NULL)
+		:part_count];
 }
 
 static int
@@ -611,6 +652,18 @@ lbox_index_max(struct lua_State *L)
 	return 1;
 }
 
+static int
+lbox_index_random(struct lua_State *L)
+{
+	if (lua_gettop(L) != 2 || lua_isnil(L, 2))
+		luaL_error(L, "Usage: index:random((uint32) rnd)");
+
+	Index *index = lua_checkindex(L, 1);
+	u32 rnd = lua_tointeger(L, 2);
+	lbox_pushtuple(L, [index random: rnd]);
+	return 1;
+}
+
 /**
  * Convert an element on Lua stack to a part of an index
  * key.
@@ -650,7 +703,10 @@ void append_key_part(struct lua_State *L, int i,
 	} else {
 		str = luaL_checklstring(L, i, &size);
 	}
-	write_varint32(tbuf, size);
+	char varint_buf[sizeof(u32) + 1];
+	size_t pack_len = (pack_varint32(varint_buf, size) -
+			   (void *) varint_buf);
+	tbuf_append(tbuf, varint_buf, pack_len);
 	tbuf_append(tbuf, str, size);
 }
 
@@ -685,19 +741,20 @@ lbox_create_iterator(struct lua_State *L)
 {
 	Index *index = lua_checkindex(L, 1);
 	int argc = lua_gettop(L);
+
+	size_t allocated_size = palloc_allocated(fiber->gc_pool);
+
 	/* Create a new iterator. */
-	enum iterator_type type;
-	int field_count;
-	void *key;
+	enum iterator_type type = ITER_ALL;
+	u32 field_count = 0;
+	const void *key = NULL;
+	u32 key_size = 0;
 	if (argc == 1 || (argc == 2 && lua_type(L, 2) == LUA_TNIL)) {
 		/*
 		 * Nothing or nil on top of the stack,
 		 * iteration over entire range from the
 		 * beginning (ITER_ALL).
 		 */
-		type = ITER_ALL;
-		field_count = 0;
-		key = NULL;
 	} else {
 		 type = luaL_checkint(L, 2);
 		 if (type >= iterator_type_MAX)
@@ -705,21 +762,25 @@ lbox_create_iterator(struct lua_State *L)
 		 /* What else do we have on the stack? */
 		 if (argc == 2 || (argc == 3 && lua_type(L, 3) == LUA_TNIL)) {
 			 /* Nothing */
-			 field_count = 0;
-			 key = NULL;
 		 } else if (argc == 3 && lua_type(L, 3) == LUA_TUSERDATA) {
 			/* Tuple. */
 			struct tuple *tuple = lua_checktuple(L, 2);
 			field_count = tuple->field_count;
 			key = tuple->data;
+			key_size = tuple->bsize;
 		} else {
 			/* Single or multi- part key. */
 			field_count = argc - 2;
-			struct tbuf *data = tbuf_alloc(fiber->gc_pool);
-			for (int i = 3; i <= argc; i++)
-				append_key_part(L, i, data,
-						index->key_def->parts[i-3].type);
+			struct tbuf *data = tbuf_new(fiber->gc_pool);
+			for (u32 i = 0; i < field_count; i++) {
+				enum field_data_type type = UNKNOWN;
+				if (i < index->key_def->part_count) {
+					type = index->key_def->parts[i].type;
+				}
+				append_key_part(L, i + 3, data, type);
+			}
 			key = data->data;
+			key_size = data->size;
 		}
 		/*
 		 * We allow partially specified keys for TREE
@@ -732,9 +793,11 @@ lbox_create_iterator(struct lua_State *L)
 				   field_count, index->key_def->part_count);
 	}
 	struct iterator *it = [index allocIterator];
-	[index initIterator: it :type :key :field_count];
-	lbox_pushiterator(L, it);
+	lbox_pushiterator(L, index, it, type, key, key_size,
+	                  field_count);
 
+	/* truncate memory used by key construction */
+	ptruncate(fiber->gc_pool, allocated_size);
 	return it;
 }
 
@@ -812,9 +875,11 @@ lbox_index_count(struct lua_State *L)
 	int argc = lua_gettop(L) - 1;
 	if (argc == 0)
 		luaL_error(L, "index.count(): one or more arguments expected");
+
 	/* preparing single or multi-part key */
-	void *key;
-	int key_part_count;
+	size_t allocated_size = palloc_allocated(fiber->gc_pool);
+	const void *key;
+	u32 key_part_count;
 	if (argc == 1 && lua_type(L, 2) == LUA_TUSERDATA) {
 		/* Searching by tuple. */
 		struct tuple *tuple = lua_checktuple(L, 2);
@@ -823,10 +888,14 @@ lbox_index_count(struct lua_State *L)
 	} else {
 		/* Single or multi- part key. */
 		key_part_count = argc;
-		struct tbuf *data = tbuf_alloc(fiber->gc_pool);
-		for (int i = 0; i < argc; ++i)
-			append_key_part(L, i + 2, data,
-					index->key_def->parts[i].type);
+		struct tbuf *data = tbuf_new(fiber->gc_pool);
+		for (u32 i = 0; i < argc; ++i) {
+			enum field_data_type type = UNKNOWN;
+			if (i < index->key_def->part_count) {
+				type = index->key_def->parts[i].type;
+			}
+			append_key_part(L, i + 2, data, type);
+		}
 		key = data->data;
 	}
 	u32 count = 0;
@@ -835,11 +904,12 @@ lbox_index_count(struct lua_State *L)
 	[index initIterator: it :ITER_EQ :key :key_part_count];
 	/* iterating over the index and counting tuples */
 	struct tuple *tuple;
-	while ((tuple = it->next(it)) != NULL) {
-		if (tuple->flags & GHOST)
-			continue;
+	while ((tuple = it->next(it)) != NULL)
 		count++;
-	}
+
+	/* truncate memory used by key construction */
+	ptruncate(fiber->gc_pool, allocated_size);
+
 	/* returning subtree size */
 	lua_pushnumber(L, count);
 	return 1;
@@ -851,6 +921,7 @@ static const struct luaL_reg lbox_index_meta[] = {
 	{"part_count", lbox_index_part_count},
 	{"min", lbox_index_min},
 	{"max", lbox_index_max},
+	{"random", lbox_index_random},
 	{"next", lbox_index_next},
 	{"iterator", lbox_index_iterator},
 	{"count", lbox_index_count},
@@ -939,6 +1010,13 @@ lua_table_to_tuple(struct lua_State *L, int index)
 			field_len = n > UINT32_MAX ? sizeof(uint64_t) : sizeof(uint32_t);
 			break;
 		}
+		case LUA_TBOOLEAN:
+		{
+			bool value = lua_toboolean(L, -1);
+			const char *str = value ? "true" : "false";
+			field_len = strlen(str) + 1;
+			break;
+		}
 		case LUA_TCDATA:
 		{
 			/* Check if we can convert. */
@@ -965,7 +1043,7 @@ lua_table_to_tuple(struct lua_State *L, int index)
 	 * the tuple is leaked.
 	 */
 	tuple->field_count = field_count;
-	u8 *pos = tuple->data;
+	void *pos = tuple->data;
 
 	/* Second go: store data in the tuple. */
 
@@ -976,27 +1054,30 @@ lua_table_to_tuple(struct lua_State *L, int index)
 		{
 			uint64_t n = lua_tonumber(L, -1);
 			if (n > UINT32_MAX) {
-				pos = memcpy(save_varint32(pos, sizeof(n)), &n,
-							   sizeof(n)) + sizeof(n);
+				pos = pack_lstr(pos, &n, sizeof(n));
 			} else {
 				uint32_t n32 = (uint32_t) n;
-				pos = memcpy(save_varint32(pos, sizeof(n32)), &n32,
-							   sizeof(n32)) + sizeof(n32);
+				pos = pack_lstr(pos, &n32, sizeof(n32));
 			}
 			break;
 		}
+		case LUA_TBOOLEAN:
+		{
+			bool value = lua_toboolean(L, -1);
+			const char *str = value ? "true" : "false";
+			pos = pack_lstr(pos, str, strlen(str) + 1);
+			break;
+		}
 		case LUA_TCDATA:
 		{
 			uint64_t n = tarantool_lua_tointeger64(L, -1);
-			pos = memcpy(save_varint32(pos, sizeof(n)), &n, sizeof(n))
-				+ sizeof(n);
+			pos = pack_lstr(pos, &n, sizeof(n));
 			break;
 		}
 		case LUA_TSTRING:
 		{
 			const char *field = lua_tolstring(L, -1, &field_len);
-			pos = memcpy(save_varint32(pos, field_len), field, field_len)
-				+ field_len;
+			pos = pack_lstr(pos, field, field_len);
 			break;
 		}
 		default:
@@ -1008,8 +1089,8 @@ lua_table_to_tuple(struct lua_State *L, int index)
 	return tuple;
 }
 
-static void
-port_add_lua_ret(struct port *port, struct lua_State *L, int index)
+static struct tuple*
+lua_totuple(struct lua_State *L, int index)
 {
 	int type = lua_type(L, index);
 	struct tuple *tuple;
@@ -1025,7 +1106,7 @@ port_add_lua_ret(struct port *port, struct lua_State *L, int index)
 		u32 num = lua_tointeger(L, index);
 		tuple = tuple_alloc(len + varint32_sizeof(len));
 		tuple->field_count = 1;
-		memcpy(save_varint32(tuple->data, len), &num, len);
+		pack_lstr(tuple->data, &num, len);
 		break;
 	}
 	case LUA_TCDATA:
@@ -1034,7 +1115,7 @@ port_add_lua_ret(struct port *port, struct lua_State *L, int index)
 		size_t len = sizeof(u64);
 		tuple = tuple_alloc(len + varint32_sizeof(len));
 		tuple->field_count = 1;
-		memcpy(save_varint32(tuple->data, len), &num, len);
+		pack_lstr(tuple->data, &num, len);
 		break;
 	}
 	case LUA_TSTRING:
@@ -1043,7 +1124,7 @@ port_add_lua_ret(struct port *port, struct lua_State *L, int index)
 		const char *str = lua_tolstring(L, index, &len);
 		tuple = tuple_alloc(len + varint32_sizeof(len));
 		tuple->field_count = 1;
-		memcpy(save_varint32(tuple->data, len), str, len);
+		pack_lstr(tuple->data, str, len);
 		break;
 	}
 	case LUA_TNIL:
@@ -1053,7 +1134,7 @@ port_add_lua_ret(struct port *port, struct lua_State *L, int index)
 		size_t len = strlen(str);
 		tuple = tuple_alloc(len + varint32_sizeof(len));
 		tuple->field_count = 1;
-		memcpy(save_varint32(tuple->data, len), str, len);
+		pack_lstr(tuple->data, str, len);
 		break;
 	}
 	case LUA_TUSERDATA:
@@ -1069,6 +1150,13 @@ port_add_lua_ret(struct port *port, struct lua_State *L, int index)
 		tnt_raise(ClientError, :ER_PROC_RET, lua_typename(L, type));
 		break;
 	}
+	return tuple;
+}
+
+static void
+port_add_lua_ret(struct port *port, struct lua_State *L, int index)
+{
+	struct tuple *tuple = lua_totuple(L, index);
 	@try {
 		port_add_tuple(port, tuple, BOX_RETURN_TUPLE);
 	} @finally {
@@ -1112,10 +1200,8 @@ static int
 lbox_process(lua_State *L)
 {
 	u32 op = lua_tointeger(L, 1); /* Get the first arg. */
-	struct tbuf req;
 	size_t sz;
-	req.data = (char *) luaL_checklstring(L, 2, &sz); /* Second arg. */
-	req.capacity = req.size = sz;
+	const void *req = luaL_checklstring(L, 2, &sz); /* Second arg. */
 	if (op == CALL) {
 		/*
 		 * We should not be doing a CALL from within a CALL.
@@ -1130,7 +1216,7 @@ lbox_process(lua_State *L)
 	size_t allocated_size = palloc_allocated(fiber->gc_pool);
 	struct port *port_lua = port_lua_create(L);
 	@try {
-		mod_process(port_lua, op, &req);
+		box_process(port_lua, op, req, sz);
 	} @finally {
 		/*
 		 * This only works as long as port_lua doesn't
@@ -1141,10 +1227,18 @@ lbox_process(lua_State *L)
 	return lua_gettop(L) - top;
 }
 
-static const struct luaL_reg boxlib[] = {
-	{"process", lbox_process},
-	{NULL, NULL}
-};
+static int
+lbox_raise(lua_State *L)
+{
+	if (lua_gettop(L) != 2)
+		luaL_error(L, "box.raise(): bad arguments");
+	uint32_t code = lua_tointeger(L, 1);
+	if (!code)
+		luaL_error(L, "box.raise(): unknown error code");
+	const char *str = lua_tostring(L, 2);
+	tnt_raise(ClientError, :code :str);
+	return 0;
+}
 
 /**
  * A helper to find a Lua function by name and put it
@@ -1189,21 +1283,22 @@ box_lua_find(lua_State *L, const char *name, const char *name_end)
 void
 box_lua_execute(struct request *request, struct port *port)
 {
-	struct tbuf *data = request->data;
+	const void **reqpos = &request->data;
+	const void *reqend = request->data + request->len;
 	lua_State *L = lua_newthread(root_L);
 	int coro_ref = luaL_ref(root_L, LUA_REGISTRYINDEX);
 	/* Request flags: not used. */
-	(void) (read_u32(data) & BOX_ALLOWED_REQUEST_FLAGS);
+	(void) (pick_u32(reqpos, reqend));
 	@try {
-		u32 field_len = read_varint32(data);
-		void *field = read_str(data, field_len); /* proc name */
+		u32 field_len;
+		/* proc name */
+		const void *field = pick_field_str(reqpos, reqend, &field_len);
 		box_lua_find(L, field, field + field_len);
 		/* Push the rest of args (a tuple). */
-		u32 nargs = read_u32(data);
+		u32 nargs = pick_u32(reqpos, reqend);
 		luaL_checkstack(L, nargs, "call: out of stack");
 		for (int i = 0; i < nargs; i++) {
-			field_len = read_varint32(data);
-			field = read_str(data, field_len);
+			field = pick_field_str(reqpos, reqend, &field_len);
 			lua_pushlstring(L, field, field_len);
 		}
 		lua_call(L, nargs, LUA_MULTRET);
@@ -1233,11 +1328,423 @@ box_index_init_iterator_types(struct lua_State *L, int idx)
 	}
 }
 
+/**
+ * Pack our BER integer into luaL_Buffer
+ */
+static void
+luaL_addvarint32(luaL_Buffer *b, u32 value)
+{
+	char buf[sizeof(u32)+1];
+	char *bufend = pack_varint32(buf, value);
+	luaL_addlstring(b, buf, bufend - buf);
+}
+
+/**
+ * Convert box.pack() format specifier to Tarantool
+ * binary protocol UPDATE opcode
+ */
+static char format_to_opcode(char format)
+{
+	switch (format) {
+	case '=': return 0;
+	case '+': return 1;
+	case '&': return 2;
+	case '^': return 3;
+	case '|': return 4;
+	case ':': return 5;
+	case '#': return 6;
+	case '!': return 7;
+	case '-': return 8;
+	default: return format;
+	}
+}
+
+/**
+ * Counterpart to @a format_to_opcode
+ */
+static char opcode_to_format(char opcode)
+{
+	switch (opcode) {
+	case 0: return '=';
+	case 1: return '+';
+	case 2: return '&';
+	case 3: return '^';
+	case 4: return '|';
+	case 5: return ':';
+	case 6: return '#';
+	case 7: return '!';
+	case 8: return '-';
+	default: return opcode;
+	}
+}
+
+static int
+luaL_packsize(struct lua_State *L, int index)
+{
+	switch (lua_type(L, index)) {
+	case LUA_TNUMBER:
+	case LUA_TCDATA:
+	case LUA_TSTRING:
+		return 1;
+	case LUA_TUSERDATA:
+	{
+		struct tuple *t = lua_istuple(L, index);
+		if (t == NULL)
+			luaL_error(L, "box.pack: unsupported type");
+		return t->field_count;
+	}
+	case LUA_TTABLE:
+	{
+		int size = 0;
+		lua_pushnil(L);
+		while (lua_next(L, index) != 0) {
+			/* Sic: use absolute index. */
+			size += luaL_packsize(L, lua_gettop(L));
+			lua_pop(L, 1);
+		}
+		return size;
+	}
+	default:
+		luaL_error(L, "box.pack: unsupported type");
+	}
+	return 0;
+}
+
+static void
+luaL_packvalue(struct lua_State *L, luaL_Buffer *b, int index)
+{
+	size_t size;
+	const char *str;
+	u32 u32buf;
+	u64 u64buf;
+	switch (lua_type(L, index)) {
+	case LUA_TNUMBER:
+		u64buf = lua_tonumber(L, index);
+		if (u64buf > UINT32_MAX) {
+			str = (char *) &u64buf;
+			size = sizeof(u64);
+		} else {
+			u32buf = (u32) u64buf;
+			str = (char *) &u32buf;
+			size = sizeof(u32);
+		}
+		break;
+	case LUA_TCDATA:
+		u64buf = tarantool_lua_tointeger64(L, index);
+		str = (char *) &u64buf;
+		size = sizeof(u64);
+		break;
+	case LUA_TSTRING:
+		str = luaL_checklstring(L, index, &size);
+		break;
+	case LUA_TUSERDATA:
+	{
+		struct tuple *tu = lua_istuple(L, index);
+		if (tu == NULL)
+			luaL_error(L, "box.pack: unsupported type");
+		luaL_addlstring(b, (char*)tu->data, tu->bsize);
+		return;
+	}
+	case LUA_TTABLE:
+	{
+		lua_pushnil(L);
+		while (lua_next(L, index) != 0) {
+			/* Sic: use absolute index. */
+			luaL_packvalue(L, b, lua_gettop(L));
+			lua_pop(L, 1);
+		}
+		return;
+	}
+	default:
+		luaL_error(L, "box.pack: unsupported type");
+		return;
+	}
+	luaL_addvarint32(b, size);
+	luaL_addlstring(b, str, size);
+}
+
+static void
+luaL_packstack(struct lua_State *L, luaL_Buffer *b, int first, int last)
+{
+	int size = 0;
+	/* sic: if arg_count is 0, first > last */
+	for (int i = first; i <= last; ++i)
+		size += luaL_packsize(L, i);
+	luaL_addlstring(b, (char *) &size, sizeof(size));
+	for (int i = first; i <= last; ++i)
+		luaL_packvalue(L, b, i);
+}
+
+
+/**
+ * To use Tarantool/Box binary protocol primitives from Lua, we
+ * need a way to pack Lua variables into a binary representation.
+ * We do it by exporting a helper function
+ *
+ * box.pack(format, args...)
+ *
+ * which takes the format, which is very similar to Perl 'pack'
+ * format, and a list of arguments, and returns a binary string
+ * which has the arguments packed according to the format.
+ *
+ * For example, a typical SELECT packet packs in Lua like this:
+ *
+ * pkt = box.pack("iiiiiip", -- pack format
+ *                         0, -- space id
+ *                         0, -- index id
+ *                         0, -- offset
+ *                         2^32, -- limit
+ *                         1, -- number of SELECT arguments
+ *                         1, -- tuple cardinality
+ *                         key); -- the key to use for SELECT
+ *
+ * @sa doc/box-protocol.txt, binary protocol description
+ * @todo: implement box.unpack(format, str), for testing purposes
+ */
+static int
+lbox_pack(struct lua_State *L)
+{
+	luaL_Buffer b;
+	const char *format = luaL_checkstring(L, 1);
+	/* first arg comes second */
+	int i = 2;
+	int nargs = lua_gettop(L);
+	u16 u16buf;
+	u32 u32buf;
+	u64 u64buf;
+	size_t size;
+	const char *str;
+
+	luaL_buffinit(L, &b);
+
+	while (*format) {
+		if (i > nargs)
+			luaL_error(L, "box.pack: argument count does not match "
+				   "the format");
+		switch (*format) {
+		case 'B':
+		case 'b':
+			/* signed and unsigned 8-bit integers */
+			u32buf = lua_tointeger(L, i);
+			if (u32buf > 0xff)
+				luaL_error(L, "box.pack: argument too big for "
+					   "8-bit integer");
+			luaL_addchar(&b, (char) u32buf);
+			break;
+		case 'S':
+		case 's':
+			/* signed and unsigned 8-bit integers */
+			u32buf = lua_tointeger(L, i);
+			if (u32buf > 0xffff)
+				luaL_error(L, "box.pack: argument too big for "
+					   "16-bit integer");
+			u16buf = (u16) u32buf;
+			luaL_addlstring(&b, (char *) &u16buf, sizeof(u16));
+			break;
+		case 'I':
+		case 'i':
+			/* signed and unsigned 32-bit integers */
+			u32buf = lua_tointeger(L, i);
+			luaL_addlstring(&b, (char *) &u32buf, sizeof(u32));
+			break;
+		case 'L':
+		case 'l':
+			/* signed and unsigned 64-bit integers */
+			u64buf = tarantool_lua_tointeger64(L, i);
+			luaL_addlstring(&b, (char *) &u64buf, sizeof(u64));
+			break;
+		case 'w':
+			/* Perl 'pack' BER-encoded integer */
+			luaL_addvarint32(&b, lua_tointeger(L, i));
+			break;
+		case 'A':
+		case 'a':
+			/* A sequence of bytes */
+			str = luaL_checklstring(L, i, &size);
+			luaL_addlstring(&b, str, size);
+			break;
+		case 'P':
+		case 'p':
+			luaL_packvalue(L, &b, i);
+			break;
+		case 'V':
+		{
+			int arg_count = luaL_checkint(L, i);
+			if (i + arg_count > nargs)
+				luaL_error(L, "box.pack: argument count does not match "
+					   "the format");
+			luaL_packstack(L, &b, i + 1, i + arg_count);
+			i += arg_count;
+			break;
+		}
+		case '=':
+			/* update tuple set foo = bar */
+		case '+':
+			/* set field += val */
+		case '-':
+			/* set field -= val */
+		case '&':
+			/* set field & =val */
+		case '|':
+			/* set field |= val */
+		case '^':
+			/* set field ^= val */
+		case ':':
+			/* splice */
+		case '#':
+			/* delete field */
+		case '!':
+			/* insert field */
+			/* field no */
+			u32buf = (u32) lua_tointeger(L, i);
+			luaL_addlstring(&b, (char *) &u32buf, sizeof(u32));
+			luaL_addchar(&b, format_to_opcode(*format));
+			break;
+		default:
+			luaL_error(L, "box.pack: unsupported pack "
+				   "format specifier '%c'", *format);
+		}
+		i++;
+		format++;
+	}
+	luaL_pushresult(&b);
+	return 1;
+}
+
+static int
+lbox_unpack(struct lua_State *L)
+{
+	size_t format_size = 0;
+	const char *format = luaL_checklstring(L, 1, &format_size);
+	const char *f = format;
+
+	size_t str_size = 0;
+	const void *str =  luaL_checklstring(L, 2, &str_size);
+	const void *end = str + str_size;
+	const void *s = str;
+
+	int i = 0;
+
+	char charbuf;
+	u8  u8buf;
+	u16 u16buf;
+	u32 u32buf;
+
+#define CHECK_SIZE(cur) if (unlikely((cur) >= end)) {	                \
+	luaL_error(L, "box.unpack('%c'): got %d bytes (expected: %d+)",	\
+		   *f, (int) (end - str), (int) 1 + ((cur) - str));	\
+}
+	while (*f) {
+		switch (*f) {
+		case 'b':
+			CHECK_SIZE(s);
+			u8buf = *(u8 *) s;
+			lua_pushnumber(L, u8buf);
+			s++;
+			break;
+		case 's':
+			CHECK_SIZE(s + 1);
+			u16buf = *(u16 *) s;
+			lua_pushnumber(L, u16buf);
+			s += 2;
+			break;
+		case 'i':
+			CHECK_SIZE(s + 3);
+			u32buf = *(u32 *) s;
+			lua_pushnumber(L, u32buf);
+			s += 4;
+			break;
+		case 'l':
+			CHECK_SIZE(s + 7);
+			luaL_pushnumber64(L, *(uint64_t*) s);
+			s += 8;
+			break;
+		case 'w':
+			/* pick_varint32 throws exception on error. */
+			u32buf = pick_varint32(&s, end);
+			lua_pushnumber(L, u32buf);
+			break;
+		case 'P':
+		case 'p':
+			/* pick_varint32 throws exception on error. */
+			u32buf = pick_varint32(&s, end);
+			CHECK_SIZE(s + u32buf - 1);
+			lua_pushlstring(L, (const char *) s, u32buf);
+			s += u32buf;
+			break;
+		case '=':
+			/* update tuple set foo = bar */
+		case '+':
+			/* set field += val */
+		case '-':
+			/* set field -= val */
+		case '&':
+			/* set field & =val */
+		case '|':
+			/* set field |= val */
+		case '^':
+			/* set field ^= val */
+		case ':':
+			/* splice */
+		case '#':
+			/* delete field */
+		case '!':
+			/* insert field */
+			CHECK_SIZE(s + 4);
+
+			/* field no */
+			u32buf = *(u32 *) s;
+
+			/* opcode */
+			charbuf = *(char *) (s + 4);
+			charbuf = opcode_to_format(charbuf);
+			if (charbuf != *f) {
+				luaL_error(L, "box.unpack('%s'): "
+					   "unexpected opcode: "
+					   "offset %d, expected '%c',"
+					   "found '%c'",
+					   format, s - str, *f, charbuf);
+			}
+
+			lua_pushnumber(L, u32buf);
+			s += 5;
+			break;
+		default:
+			luaL_error(L, "box.unpack: unsupported "
+				   "format specifier '%c'", *f);
+		}
+		i++;
+		f++;
+	}
+
+	assert(s <= end);
+
+	if (s != end) {
+		luaL_error(L, "box.unpack('%s'): too many bytes: "
+			   "unpacked %d, total %d",
+			   format, s - str, str_size);
+	}
+
+	return i;
+
+#undef CHECK_SIZE
+}
+
+static const struct luaL_reg boxlib[] = {
+	{"process", lbox_process},
+	{"raise", lbox_raise},
+	{"pack", lbox_pack},
+	{"unpack", lbox_unpack},
+	{NULL, NULL}
+};
+
 void
 mod_lua_init(struct lua_State *L)
 {
 	/* box, box.tuple */
 	tarantool_lua_register_type(L, tuplelib_name, lbox_tuple_meta);
+	luaL_register(L, tuplelib_name, lbox_tuplelib);
+	lua_pop(L, 1);
 	luaL_register(L, "box", boxlib);
 	lua_pop(L, 1);
 	/* box.index */
