@@ -34,7 +34,6 @@
 #include "port.h"
 #include "box_lua.h"
 #include <errinj.h>
-#include <tbuf.h>
 #include <pickle.h>
 #include <fiber.h>
 #include <rope.h>
@@ -42,69 +41,63 @@
 STRS(requests, REQUESTS);
 STRS(update_op_codes, UPDATE_OP_CODES);
 
-static void
-read_key(struct tbuf *data, void **key_ptr, u32 *key_part_count_ptr)
+static const void *
+read_key(const void **reqpos, const void *reqend, u32 *key_part_count)
 {
-	void *key = NULL;
-	u32 key_part_count = read_u32(data);
-	if (key_part_count) {
-		key = read_field(data);
-		/* advance remaining fields of a key */
-		for (int i = 1; i < key_part_count; i++)
-			read_field(data);
-	}
-
-	*key_ptr = key;
-	*key_part_count_ptr = key_part_count;
+	*key_part_count = pick_u32(reqpos, reqend);
+	const void *key = *key_part_count ? *reqpos : NULL;
+	/* Advance remaining fields of a key */
+	for (u32 i = 0; i < *key_part_count; i++)
+		pick_field(reqpos, reqend);
+	return key;
 }
 
 static struct space *
-read_space(struct tbuf *data)
+read_space(const void **reqpos, const void *reqend)
 {
-	u32 space_no = read_u32(data);
+	u32 space_no = pick_u32(reqpos, reqend);
 	return space_find(space_no);
 }
 
+enum dup_replace_mode
+dup_replace_mode(uint32_t flags)
+{
+	return flags & BOX_ADD ? DUP_INSERT :
+		flags & BOX_REPLACE ?
+		DUP_REPLACE : DUP_REPLACE_OR_INSERT;
+}
+
 static void
 execute_replace(struct request *request, struct txn *txn)
 {
-	struct tbuf *data = request->data;
-	txn_add_redo(txn, request->type, data);
-	struct space *sp = read_space(data);
-	request->flags |= read_u32(data) & BOX_ALLOWED_REQUEST_FLAGS;
-	size_t field_count = read_u32(data);
+	txn_add_redo(txn, request->type, request->data, request->len);
+	const void **reqpos = &request->data;
+	const void *reqend = request->data + request->len;
+	struct space *sp = read_space(reqpos, reqend);
+	request->flags |= (pick_u32(reqpos, reqend) &
+			   BOX_ALLOWED_REQUEST_FLAGS);
+	size_t field_count = pick_u32(reqpos, reqend);
 
 	if (field_count == 0)
 		tnt_raise(IllegalParams, :"tuple field count is 0");
 
-	if (data->size == 0 || data->size != valid_tuple(data, field_count))
+	size_t tuple_len = reqend - *reqpos;
+	if (tuple_len != valid_tuple(*reqpos, reqend, field_count))
 		tnt_raise(IllegalParams, :"incorrect tuple length");
 
-	txn->new_tuple = tuple_alloc(data->size);
-	tuple_ref(txn->new_tuple, 1);
-	txn->new_tuple->field_count = field_count;
-	memcpy(txn->new_tuple->data, data->data, data->size);
+	struct tuple *new_tuple = tuple_alloc(tuple_len);
+	new_tuple->field_count = field_count;
+	memcpy(new_tuple->data, *reqpos, tuple_len);
 
-	/* Try to find tuple by primary key */
-	Index *pk = space_index(sp, 0);
+	@try {
+		space_validate_tuple(sp, new_tuple);
+		enum dup_replace_mode mode = dup_replace_mode(request->flags);
+		txn_replace(txn, sp, NULL, new_tuple, mode);
 
-	/* Check to see if the tuple has a sufficient number of fields. */
-	if (unlikely(txn->new_tuple->field_count < sp->max_fieldno)) {
-		tnt_raise(IllegalParams, :"tuple must have all indexed fields");
+	} @catch (tnt_Exception *e) {
+		tuple_free(new_tuple);
+		@throw;
 	}
-
-	/* lookup old_tuple only when we have enough fields in new_tuple */
-	struct tuple *old_tuple = [pk findByTuple: txn->new_tuple];
-
-	if (request->flags & BOX_ADD && old_tuple != NULL)
-		tnt_raise(ClientError, :ER_TUPLE_FOUND);
-
-	if (request->flags & BOX_REPLACE && old_tuple == NULL)
-		tnt_raise(ClientError, :ER_TUPLE_NOT_FOUND);
-
-	space_validate(sp, old_tuple, txn->new_tuple);
-
-	txn_add_undo(txn, sp, old_tuple, txn->new_tuple);
 }
 
 /** {{{ UPDATE request implementation.
@@ -172,7 +165,7 @@ execute_replace(struct request *request, struct txn *txn)
 /** Argument of SET operation. */
 struct op_set_arg {
 	u32 length;
-	void *value;
+	const void *value;
 };
 
 /** Argument of ADD, AND, XOR, OR operations. */
@@ -186,10 +179,10 @@ struct op_arith_arg {
 
 /** Argument of SPLICE. */
 struct op_splice_arg {
-	i32 offset;	/** splice position */
-	i32 cut_length; /** cut this many bytes. */
-	void *paste;      /** paste what? */
-	i32 paste_length; /** paste this many bytes. */
+	i32 offset;	   /** splice position */
+	i32 cut_length;    /** cut this many bytes. */
+	const void *paste; /** paste what? */
+	i32 paste_length;  /** paste this many bytes. */
 
 	/** Offset of the tail in the old field */
 	i32 tail_offset;
@@ -207,7 +200,7 @@ struct update_field;
 struct update_op;
 
 typedef void (*init_op_func)(struct rope *rope, struct update_op *op);
-typedef void (*do_op_func)(union update_op_arg *arg, void *in, void *out);
+typedef void (*do_op_func)(union update_op_arg *arg, const void *in, void *out);
 
 /** A set of functions and properties to initialize and do an op. */
 struct update_op_meta {
@@ -236,9 +229,9 @@ struct update_field {
 	/** UPDATE operations against the first field in the range. */
 	struct op_list ops;
 	/** Points at start of field *data* in the old tuple. */
-	void *old;
+	const void *old;
 	/** End of the old field. */
-	void *tail;
+	const void *tail;
 	/**
 	 * Length of the "tail" in the old tuple from end
 	 * of old data to the beginning of the field in the
@@ -249,7 +242,7 @@ struct update_field {
 
 static void
 update_field_init(struct update_field *field,
-		  void *old, u32 old_len, u32 tail_len)
+		  const void *old, u32 old_len, u32 tail_len)
 {
 	STAILQ_INIT(&field->ops);
 	field->old = old;
@@ -282,14 +275,14 @@ op_adjust_field_no(struct update_op *op, u32 field_max)
 
 
 static void
-do_update_op_set(struct op_set_arg *arg, void *in __attribute__((unused)),
+do_update_op_set(struct op_set_arg *arg, const void *in __attribute__((unused)),
 		 void *out)
 {
 	memcpy(out, arg->value, arg->length);
 }
 
 static void
-do_update_op_add(struct op_arith_arg *arg, void *in, void *out)
+do_update_op_add(struct op_arith_arg *arg, const void *in, void *out)
 {
 	if (arg->val_size == sizeof(i32))
 		*(i32 *)out = *(i32 *)in + arg->i32_val;
@@ -298,7 +291,7 @@ do_update_op_add(struct op_arith_arg *arg, void *in, void *out)
 }
 
 static void
-do_update_op_subtract(struct op_arith_arg *arg, void *in, void *out)
+do_update_op_subtract(struct op_arith_arg *arg, const void *in, void *out)
 {
 	if (arg->val_size == sizeof(i32))
 		*(i32 *)out = *(i32 *)in - arg->i32_val;
@@ -307,7 +300,7 @@ do_update_op_subtract(struct op_arith_arg *arg, void *in, void *out)
 }
 
 static void
-do_update_op_and(struct op_arith_arg *arg, void *in, void *out)
+do_update_op_and(struct op_arith_arg *arg, const void *in, void *out)
 {
 	if (arg->val_size == sizeof(i32))
 		*(i32 *)out = *(i32 *)in & arg->i32_val;
@@ -316,7 +309,7 @@ do_update_op_and(struct op_arith_arg *arg, void *in, void *out)
 }
 
 static void
-do_update_op_xor(struct op_arith_arg *arg, void *in, void *out)
+do_update_op_xor(struct op_arith_arg *arg, const void *in, void *out)
 {
 	if (arg->val_size == sizeof(i32))
 		*(i32 *)out = *(i32 *)in ^ arg->i32_val;
@@ -325,7 +318,7 @@ do_update_op_xor(struct op_arith_arg *arg, void *in, void *out)
 }
 
 static void
-do_update_op_or(struct op_arith_arg *arg, void *in, void *out)
+do_update_op_or(struct op_arith_arg *arg, const void *in, void *out)
 {
 	if (arg->val_size == sizeof(i32))
 		*(i32 *)out = *(i32 *)in | arg->i32_val;
@@ -334,7 +327,7 @@ do_update_op_or(struct op_arith_arg *arg, void *in, void *out)
 }
 
 static void
-do_update_op_splice(struct op_splice_arg *arg, void *in, void *out)
+do_update_op_splice(struct op_splice_arg *arg, const void *in, void *out)
 {
 	memcpy(out, in, arg->offset);           /* copy field head. */
 	out += arg->offset;
@@ -344,8 +337,9 @@ do_update_op_splice(struct op_splice_arg *arg, void *in, void *out)
 }
 
 static void
-do_update_op_insert(struct op_set_arg *arg, void *in __attribute__((unused)),
-		 void *out)
+do_update_op_insert(struct op_set_arg *arg,
+		    const void *in __attribute__((unused)),
+		    void *out)
 {
 	memcpy(out, arg->value, arg->length);
 }
@@ -434,21 +428,13 @@ init_update_op_splice(struct rope *rope, struct update_op *op)
 	struct update_field *field = rope_extract(rope, op->field_no);
 
 	u32 field_len = update_field_len(field);
-	struct tbuf operands = {
-		.capacity = op->arg.set.length,
-		.size = op->arg.set.length,
-		.data = op->arg.set.value,
-		.pool = NULL
-	};
+
 	struct op_splice_arg *arg = &op->arg.splice;
+	const void *value = op->arg.set.value;
+	const void *end = value + op->arg.set.length;
 
 	/* Read the offset. */
-	void *offset_field = read_field(&operands);
-	u32 len = load_varint32(&offset_field);
-	if (len != sizeof(i32))
-		tnt_raise(IllegalParams, :"SPLICE offset");
-	/* Sic: overwrite of op->arg.set.length. */
-	arg->offset = *(i32 *)offset_field;
+	arg->offset = pick_field_u32(&value, end);
 	if (arg->offset < 0) {
 		if (-arg->offset > field_len)
 			tnt_raise(ClientError, :ER_SPLICE,
@@ -460,11 +446,7 @@ init_update_op_splice(struct rope *rope, struct update_op *op)
 	assert(arg->offset >= 0 && arg->offset <= field_len);
 
 	/* Read the cut length. */
-	void *length_field = read_field(&operands);
-	len = load_varint32(&length_field);
-	if (len != sizeof(i32))
-		tnt_raise(IllegalParams, :"SPLICE length");
-	arg->cut_length = *(i32 *)length_field;
+	arg->cut_length = pick_field_u32(&value, end);
 	if (arg->cut_length < 0) {
 		if (-arg->cut_length > (field_len - arg->offset))
 			arg->cut_length = 0;
@@ -475,16 +457,14 @@ init_update_op_splice(struct rope *rope, struct update_op *op)
 	}
 
 	/* Read the paste. */
-	void *paste_field = read_field(&operands);
-	arg->paste_length = load_varint32(&paste_field);
-	arg->paste = paste_field;
+	arg->paste = pick_field_str(&value, end, (u32 *) &arg->paste_length);
 
 	/* Fill tail part */
 	arg->tail_offset = arg->offset + arg->cut_length;
 	arg->tail_length = field_len - arg->tail_offset;
 
 	/* Check that the operands are fully read. */
-	if (operands.size != 0)
+	if (value != end)
 		tnt_raise(IllegalParams, :"field splice format error");
 
 	/* Record the new field length. */
@@ -528,8 +508,8 @@ update_field_split(void *data, size_t size __attribute__((unused)),
 					   sizeof(struct update_field));
 	assert(offset > 0 && prev->tail_len > 0);
 
-	void *field = prev->tail;
-	void *end = field + prev->tail_len;
+	const void *field = prev->tail;
+	const void *end = field + prev->tail_len;
 
 	prev->tail_len = tuple_range_size(&field, end, offset - 1);
 	u32 field_len = load_varint32(&field);
@@ -555,8 +535,8 @@ update_create_rope(struct update_op *op, struct update_op *op_end,
 
 	struct update_field *first = palloc(fiber->gc_pool,
 					    sizeof(struct update_field));
-	void *field = tuple->data;
-	void *end = tuple->data + tuple->bsize;
+	const void *field = tuple->data;
+	const void *end = tuple->data + tuple->bsize;
 	u32 field_len = load_varint32(&field);
 	update_field_init(first, field, field_len, end - field - field_len);
 
@@ -575,7 +555,7 @@ update_calc_new_tuple_length(struct rope *rope)
 	struct rope_iter it;
 	struct rope_node *node;
 
-	rope_iter_init(&it, rope);
+	rope_iter_create(&it, rope);
 	for (node = rope_iter_start(&it); node; node = rope_iter_next(&it)) {
 		struct update_field *field = rope_leaf_data(node);
 		u32 field_len = update_field_len(field);
@@ -600,18 +580,18 @@ do_update_ops(struct rope *rope, struct tuple *new_tuple)
 	struct rope_iter it;
 	struct rope_node *node;
 
-	rope_iter_init(&it, rope);
+	rope_iter_create(&it, rope);
 	for (node = rope_iter_start(&it); node; node = rope_iter_next(&it)) {
 
 		struct update_field *field = rope_leaf_data(node);
 		u32 field_count = rope_leaf_size(node);
 		u32 field_len = update_field_len(field);
 
-		new_data = save_varint32(new_data, field_len);
+		new_data = pack_varint32(new_data, field_len);
 
-		void *old_field = field->old;
+		const void *old_field = field->old;
 		void *new_field = (STAILQ_EMPTY(&field->ops) ?
-				   old_field : new_data);
+				   (void*) old_field : new_data);
 		struct update_op *op;
 		STAILQ_FOREACH(op, &field->ops, next) {
 			/*
@@ -660,7 +640,7 @@ do_update_ops(struct rope *rope, struct tuple *new_tuple)
 }
 
 static struct update_op *
-update_read_ops(struct tbuf *data, u32 op_cnt)
+update_read_ops(const void **reqpos, const void *reqend, u32 op_cnt)
 {
 	if (op_cnt > BOX_UPDATE_OP_CNT_MAX)
 		tnt_raise(IllegalParams, :"too many operations for update");
@@ -672,18 +652,18 @@ update_read_ops(struct tbuf *data, u32 op_cnt)
 	struct update_op *op = ops, *ops_end = ops + op_cnt;
 	for (; op < ops_end; op++) {
 		/* Read operation */
-		op->field_no = read_u32(data);
-		op->opcode = read_u8(data);
+		op->field_no = pick_u32(reqpos, reqend);
+		op->opcode = pick_u8(reqpos, reqend);
 
 		if (op->opcode >= UPDATE_OP_MAX)
 			tnt_raise(ClientError, :ER_UNKNOWN_UPDATE_OP);
 		op->meta = &update_op_meta[op->opcode];
 
-		op->arg.set.value = read_field(data);
+		op->arg.set.value = pick_field(reqpos, reqend);
 		op->arg.set.length = load_varint32(&op->arg.set.value);
 	}
 	/* Check the remainder length, the request must be fully read. */
-	if (data->size != 0)
+	if (*reqpos != reqend)
 		tnt_raise(IllegalParams, :"can't unpack request");
 	return ops;
 }
@@ -691,37 +671,42 @@ update_read_ops(struct tbuf *data, u32 op_cnt)
 static void
 execute_update(struct request *request, struct txn *txn)
 {
-	struct tbuf *data = request->data;
-	txn_add_redo(txn, request->type, data);
-	struct space *sp = read_space(data);
-	request->flags |= read_u32(data) & BOX_ALLOWED_REQUEST_FLAGS;
-
+	txn_add_redo(txn, request->type, request->data, request->len);
+	const void **reqpos = &request->data;
+	const void *reqend = request->data + request->len;
+	struct space *sp = read_space(reqpos, reqend);
+	request->flags |= (pick_u32(reqpos, reqend) &
+			   BOX_ALLOWED_REQUEST_FLAGS);
 	/* Parse UPDATE request. */
-	/** Search key */
-	void *key;
-	/** Search key part count. */
+	/** Search key  and key part count. */
 	u32 key_part_count;
-
-	read_key(data, &key, &key_part_count);
+	const void *key = read_key(reqpos, reqend, &key_part_count);
 
 	Index *pk = space_index(sp, 0);
 	/* Try to find the tuple by primary key. */
 	struct tuple *old_tuple = [pk findByKey :key :key_part_count];
 
-	if (old_tuple != NULL) {
-		/* number of operations */
-		u32 op_cnt = read_u32(data);
-		struct update_op *ops = update_read_ops(data, op_cnt);
-		struct rope *rope = update_create_rope(ops, ops + op_cnt,
-						       old_tuple);
-		/* allocate new tuple */
-		size_t new_tuple_len = update_calc_new_tuple_length(rope);
-		txn->new_tuple = tuple_alloc(new_tuple_len);
-		tuple_ref(txn->new_tuple, 1);
-		do_update_ops(rope, txn->new_tuple);
-		space_validate(sp, old_tuple, txn->new_tuple);
+	if (old_tuple == NULL)
+		return;
+
+	/* number of operations */
+	u32 op_cnt = pick_u32(reqpos, reqend);
+	struct update_op *ops = update_read_ops(reqpos, reqend, op_cnt);
+	struct rope *rope = update_create_rope(ops, ops + op_cnt,
+					       old_tuple);
+	/* Allocate a new tuple. */
+	size_t new_tuple_len = update_calc_new_tuple_length(rope);
+	struct tuple *new_tuple = tuple_alloc(new_tuple_len);
+
+	@try {
+		do_update_ops(rope, new_tuple);
+		space_validate_tuple(sp, new_tuple);
+		txn_replace(txn, sp, old_tuple, new_tuple, DUP_INSERT);
+
+	} @catch (tnt_Exception *e) {
+		tuple_free(new_tuple);
+		@throw;
 	}
-	txn_add_undo(txn, sp, old_tuple, txn->new_tuple);
 }
 
 /** }}} */
@@ -729,13 +714,14 @@ execute_update(struct request *request, struct txn *txn)
 static void
 execute_select(struct request *request, struct port *port)
 {
-	struct tbuf *data = request->data;
-	struct space *sp = read_space(data);
-	u32 index_no = read_u32(data);
+	const void **reqpos = &request->data;
+	const void *reqend = request->data + request->len;
+	struct space *sp = read_space(reqpos, reqend);
+	u32 index_no = pick_u32(reqpos, reqend);
 	Index *index = index_find(sp, index_no);
-	u32 offset = read_u32(data);
-	u32 limit = read_u32(data);
-	u32 count = read_u32(data);
+	u32 offset = pick_u32(reqpos, reqend);
+	u32 limit = pick_u32(reqpos, reqend);
+	u32 count = pick_u32(reqpos, reqend);
 	if (count == 0)
 		tnt_raise(IllegalParams, :"tuple count must be positive");
 
@@ -751,17 +737,13 @@ execute_select(struct request *request, struct port *port)
 
 		/* read key */
 		u32 key_part_count;
-		void *key;
-		read_key(data, &key, &key_part_count);
+		const void *key = read_key(reqpos, reqend, &key_part_count);
 
 		struct iterator *it = index->position;
 		[index initIterator: it :ITER_EQ :key :key_part_count];
 
 		struct tuple *tuple;
 		while ((tuple = it->next(it)) != NULL) {
-			if (tuple->flags & GHOST)
-				continue;
-
 			if (offset > 0) {
 				offset--;
 				continue;
@@ -773,28 +755,33 @@ execute_select(struct request *request, struct port *port)
 				break;
 		}
 	}
-	if (data->size != 0)
+	if (*reqpos != reqend)
 		tnt_raise(IllegalParams, :"can't unpack request");
 }
 
 static void
 execute_delete(struct request *request, struct txn *txn)
 {
-	struct tbuf *data = request->data;
 	u32 type = request->type;
-	txn_add_redo(txn, type, data);
-	struct space *sp = read_space(data);
-	if (type == DELETE)
-		request->flags |= read_u32(data) & BOX_ALLOWED_REQUEST_FLAGS;
+	txn_add_redo(txn, type, request->data, request->len);
+	const void **reqpos = &request->data;
+	const void *reqend = request->data + request->len;
+	struct space *sp = read_space(reqpos, reqend);
+	if (type == DELETE) {
+		request->flags |= pick_u32(reqpos, reqend) &
+			BOX_ALLOWED_REQUEST_FLAGS;
+	}
 	/* read key */
 	u32 key_part_count;
-	void *key;
-	read_key(data, &key, &key_part_count);
+	const void *key = read_key(reqpos, reqend, &key_part_count);
 	/* Try to find tuple by primary key */
 	Index *pk = space_index(sp, 0);
 	struct tuple *old_tuple = [pk findByKey :key :key_part_count];
 
-	txn_add_undo(txn, sp, old_tuple, NULL);
+	if (old_tuple == NULL)
+		return;
+
+	txn_replace(txn, sp, old_tuple, NULL, DUP_REPLACE_OR_INSERT);
 }
 
 /** To collects stats, we need a valid request type.
@@ -818,7 +805,7 @@ request_name(u32 type)
 }
 
 struct request *
-request_create(u32 type, struct tbuf *data)
+request_create(u32 type, const void *data, u32 len)
 {
 	if (request_check_type(type)) {
 		say_error("Unsupported request = %" PRIi32 "", type);
@@ -829,6 +816,7 @@ request_create(u32 type, struct tbuf *data)
 	struct request *request = palloc(fiber->gc_pool, sizeof(struct request));
 	request->type = type;
 	request->data = data;
+	request->len = len;
 	request->flags = 0;
 	return request;
 }
