@@ -37,7 +37,6 @@
 #include "errcode.h"
 #include "fiber.h"
 #include "say.h"
-#include "tbuf.h"
 #include "box/box.h"
 #include "box/port.h"
 #include "box/tuple.h"
@@ -331,6 +330,9 @@ iproto_queue_init(struct iproto_queue *i_queue,
 	rlist_create(&i_queue->fiber_cache);
 }
 
+static inline uint32_t
+iproto_session_id(struct iproto_session *session);
+
 /** A handler to process all queued requests. */
 static void
 iproto_queue_handler(va_list ap)
@@ -340,6 +342,7 @@ iproto_queue_handler(va_list ap)
 restart:
 	while (iproto_dequeue_request(i_queue, &request)) {
 
+		fiber_set_sid(fiber, iproto_session_id(request.session));
 		request.process(&request);
 	}
 	iproto_cache_fiber(&request_queue);
@@ -395,6 +398,10 @@ SLIST_HEAD(, iproto_session) iproto_session_cache =
  * A session is idle when the client is gone
  * and there are no outstanding requests in the request queue.
  * An idle session can be safely garbage collected.
+ * Note: a session only becomes idle after iproto_session_shutdown(),
+ * which closes the fd.  This is why here the check is for
+ * evio_is_active() (false if fd is closed), not ev_is_active()
+ * (false if event is not started).
  */
 static inline bool
 iproto_session_is_idle(struct iproto_session *session)
@@ -404,6 +411,12 @@ iproto_session_is_idle(struct iproto_session *session)
 		ibuf_size(&session->iobuf[1]->in) == 0;
 }
 
+static inline uint32_t
+iproto_session_id(struct iproto_session *session)
+{
+	return session->sid;
+}
+
 static void
 iproto_session_on_input(struct ev_io *watcher,
 			int revents __attribute__((unused)));
@@ -430,6 +443,8 @@ iproto_session_create(const char *name, int fd, box_process_func *param)
 	} else {
 		session = SLIST_FIRST(&iproto_session_cache);
 		SLIST_REMOVE_HEAD(&iproto_session_cache, next_in_cache);
+		assert(session->input.fd == -1);
+		assert(session->output.fd == -1);
 	}
 	session->handler = param;
 	ev_io_init(&session->input, iproto_session_on_input, fd, EV_READ);
@@ -447,19 +462,13 @@ static inline void
 iproto_session_destroy(struct iproto_session *session)
 {
 	assert(iproto_session_is_idle(session));
+	assert(!evio_is_active(&session->output));
 	session_destroy(session->sid); /* Never throws. No-op if sid is 0. */
 	iobuf_delete(session->iobuf[0]);
 	iobuf_delete(session->iobuf[1]);
 	SLIST_INSERT_HEAD(&iproto_session_cache, session, next_in_cache);
 }
 
-static inline void
-iproto_session_gc(struct iproto_session *session)
-{
-	if (iproto_session_is_idle(session))
-		iproto_session_destroy(session);
-}
-
 static inline void
 iproto_session_shutdown(struct iproto_session *session)
 {
@@ -473,13 +482,18 @@ iproto_session_shutdown(struct iproto_session *session)
 	 */
 	session->iobuf[0]->in.end -= session->parse_size;
 	/*
-	 * If the session is not idle it will
-	 * be destroyed only after all its requests
-	 * are processed (and dropped).
+	 * If the session is not idle, it is destroyed
+	 * after the last request is handled. Otherwise,
+	 * queue a separate request to run on_disconnect()
+	 * trigger and destroy the session.
+	 * Sic: the check is mandatory to not destroy a session
+	 * twice.
          */
-	iproto_enqueue_request(&request_queue, session,
-			       session->iobuf[0], &dummy_header,
-			       iproto_process_disconnect);
+	if (iproto_session_is_idle(session)) {
+		iproto_enqueue_request(&request_queue, session,
+				       session->iobuf[0], &dummy_header,
+				       iproto_process_disconnect);
+	}
 }
 
 static inline void
@@ -596,6 +610,7 @@ iproto_session_on_input(struct ev_io *watcher,
 {
 	struct iproto_session *session = watcher->data;
 	int fd = session->input.fd;
+	assert(fd >= 0);
 
 	@try {
 		/* Ensure we have sufficient space for the next round.  */
@@ -621,6 +636,12 @@ iproto_session_on_input(struct ev_io *watcher,
 		session->parse_size += nrd;
 		/* Enqueue all requests which are fully read up. */
 		iproto_enqueue_batch(session, in, fd);
+		/*
+		 * Keep reading input, as long as the socket
+		 * supplies data.
+		 */
+		if (!ev_is_active(&session->input))
+			ev_feed_event(&session->input, EV_READ);
 	} @catch (tnt_Exception *e) {
 		[e log];
 		iproto_session_shutdown(session);
@@ -734,13 +755,11 @@ iproto_reply(struct port_iproto *port, box_process_func callback,
 		return iproto_reply_ping(out, header);
 
 	/* Make request body point to iproto data */
-	struct tbuf body = {
-		.size = header->len, .capacity = header->len,
-		.data = (char *) &header[1], .pool = fiber->gc_pool
-	};
+	void *body = (char *) &header[1];
 	port_iproto_init(port, out, header);
 	@try {
-		callback((struct port *) port, header->msg_code, &body);
+		callback((struct port *) port, header->msg_code,
+			 body, header->len);
 	} @catch (ClientError *e) {
 		if (port->reply.found)
 			obuf_rollback_to_svp(out, &port->svp);
@@ -759,7 +778,6 @@ iproto_process_request(struct iproto_request *request)
 		if (unlikely(! evio_is_active(&session->output)))
 			return;
 
-		fiber_set_sid(fiber, session->sid);
 		iproto_reply(&port, *session->handler,
 			     &iobuf->out, header);
 
@@ -769,7 +787,8 @@ iproto_process_request(struct iproto_request *request)
 			ev_feed_event(&session->output, EV_WRITE);
 	} @finally {
 		iobuf->in.pos += sizeof(*header) + header->len;
-		iproto_session_gc(session);
+		if (iproto_session_is_idle(session))
+			iproto_session_destroy(session);
 	}
 }
 
@@ -788,7 +807,11 @@ iproto_process_connect(struct iproto_request *request)
 		session->sid = session_create(fd);
 	} @catch (ClientError *e) {
 		iproto_reply_error(&iobuf->out, request->header, e);
-		iproto_flush(iobuf, fd, &session->write_pos);
+		@try {
+			iproto_flush(iobuf, fd, &session->write_pos);
+		} @catch (tnt_Exception *e) {
+			[e log];
+		}
 		iproto_session_shutdown(session);
 		return;
 	} @catch (tnt_Exception *e) {
@@ -797,6 +820,12 @@ iproto_process_connect(struct iproto_request *request)
 		iproto_session_shutdown(session);
 		return;
 	}
+	/*
+	 * Connect is synchronous, so no one could have been
+	 * messing up with the session while it was in
+	 * progress.
+	 */
+	assert(evio_is_active(&session->input));
 	/* Handshake OK, start reading input. */
 	ev_feed_event(&session->input, EV_READ);
 }
@@ -806,7 +835,7 @@ iproto_process_disconnect(struct iproto_request *request)
 {
 	fiber_set_sid(fiber, request->session->sid);
 	/* Runs the trigger, which may yield. */
-	iproto_session_gc(request->session);
+	iproto_session_destroy(request->session);
 }
 
 /** }}} */
