@@ -47,8 +47,8 @@ memcached_dispatch(struct ev_io *coio, struct iobuf *iobuf)
 	int cs;
 	char *p, *pe;
 	char *fstart;
-	struct tbuf *keys = tbuf_alloc(fiber->gc_pool);
-	void *key;
+	struct tbuf *keys = tbuf_new(fiber->gc_pool);
+	const void *key;
 	bool append, show_cas;
 	int incr_sign;
 	u64 cas, incr;
@@ -151,8 +151,7 @@ tr15:
 			fstart = p;
 			for (; p < pe && *p != ' ' && *p != '\r' && *p != '\n'; p++);
 			if ( *p == ' ' || *p == '\r' || *p == '\n') {
-				write_varint32(keys, p - fstart);
-				tbuf_append(keys, fstart, p - fstart);
+				tbuf_store_field(keys, fstart, p - fstart);
 				keys_count++;
 				p--;
 			} else
@@ -163,7 +162,7 @@ st6:
 	if ( ++p == pe )
 		goto _test_eof6;
 case 6:
-#line 167 "src/memcached-grammar.m"
+#line 166 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st7;
 	goto st0;
@@ -184,21 +183,21 @@ st8:
 	if ( ++p == pe )
 		goto _test_eof8;
 case 8:
-#line 188 "src/memcached-grammar.m"
+#line 187 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto tr18;
 	if ( 48 <= (*p) && (*p) <= 57 )
 		goto st8;
 	goto st0;
 tr18:
-#line 246 "src/memcached-grammar.rl"
+#line 245 "src/memcached-grammar.rl"
 	{flags = natoq(fstart, p);}
 	goto st9;
 st9:
 	if ( ++p == pe )
 		goto _test_eof9;
 case 9:
-#line 202 "src/memcached-grammar.m"
+#line 201 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st9;
 	if ( 48 <= (*p) && (*p) <= 57 )
@@ -212,14 +211,14 @@ st10:
 	if ( ++p == pe )
 		goto _test_eof10;
 case 10:
-#line 216 "src/memcached-grammar.m"
+#line 215 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto tr22;
 	if ( 48 <= (*p) && (*p) <= 57 )
 		goto st10;
 	goto st0;
 tr22:
-#line 239 "src/memcached-grammar.rl"
+#line 238 "src/memcached-grammar.rl"
 	{
 			exptime = natoq(fstart, p);
 			if (exptime > 0 && exptime <= 60*60*24*30)
@@ -230,7 +229,7 @@ st11:
 	if ( ++p == pe )
 		goto _test_eof11;
 case 11:
-#line 234 "src/memcached-grammar.m"
+#line 233 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st11;
 	if ( 48 <= (*p) && (*p) <= 57 )
@@ -244,7 +243,7 @@ st12:
 	if ( ++p == pe )
 		goto _test_eof12;
 case 12:
-#line 248 "src/memcached-grammar.m"
+#line 247 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr26;
 		case 13: goto tr27;
@@ -254,16 +253,17 @@ case 12:
 		goto st12;
 	goto st0;
 tr26:
-#line 247 "src/memcached-grammar.rl"
+#line 246 "src/memcached-grammar.rl"
 	{bytes = natoq(fstart, p);}
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -288,7 +288,7 @@ tr26:
 		}
 #line 68 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple != NULL && !expired(tuple))
 				obuf_dup(out, "NOT_STORED\r\n", 12);
@@ -299,12 +299,13 @@ tr26:
 tr30:
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -329,7 +330,7 @@ tr30:
 		}
 #line 68 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple != NULL && !expired(tuple))
 				obuf_dup(out, "NOT_STORED\r\n", 12);
@@ -342,12 +343,13 @@ tr39:
 	{ noreply = true; }
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -372,7 +374,7 @@ tr39:
 		}
 #line 68 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple != NULL && !expired(tuple))
 				obuf_dup(out, "NOT_STORED\r\n", 12);
@@ -381,16 +383,17 @@ tr39:
 		}
 	goto st197;
 tr58:
-#line 247 "src/memcached-grammar.rl"
+#line 246 "src/memcached-grammar.rl"
 	{bytes = natoq(fstart, p);}
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -416,17 +419,17 @@ tr58:
 #line 97 "src/memcached-grammar.rl"
 	{
 			struct tbuf *b;
-			void *value;
+			const void *value;
 			u32 value_len;
 
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL) {
 				obuf_dup(out, "NOT_STORED\r\n", 12);
 			} else {
 				value = tuple_field(tuple, 3);
 				value_len = load_varint32(&value);
-				b = tbuf_alloc(fiber->gc_pool);
+				b = tbuf_new(fiber->gc_pool);
 				if (append) {
 					tbuf_append(b, value, value_len);
 					tbuf_append(b, data, bytes);
@@ -444,12 +447,13 @@ tr58:
 tr62:
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -475,17 +479,17 @@ tr62:
 #line 97 "src/memcached-grammar.rl"
 	{
 			struct tbuf *b;
-			void *value;
+			const void *value;
 			u32 value_len;
 
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL) {
 				obuf_dup(out, "NOT_STORED\r\n", 12);
 			} else {
 				value = tuple_field(tuple, 3);
 				value_len = load_varint32(&value);
-				b = tbuf_alloc(fiber->gc_pool);
+				b = tbuf_new(fiber->gc_pool);
 				if (append) {
 					tbuf_append(b, value, value_len);
 					tbuf_append(b, data, bytes);
@@ -505,12 +509,13 @@ tr71:
 	{ noreply = true; }
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -536,17 +541,17 @@ tr71:
 #line 97 "src/memcached-grammar.rl"
 	{
 			struct tbuf *b;
-			void *value;
+			const void *value;
 			u32 value_len;
 
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL) {
 				obuf_dup(out, "NOT_STORED\r\n", 12);
 			} else {
 				value = tuple_field(tuple, 3);
 				value_len = load_varint32(&value);
-				b = tbuf_alloc(fiber->gc_pool);
+				b = tbuf_new(fiber->gc_pool);
 				if (append) {
 					tbuf_append(b, value, value_len);
 					tbuf_append(b, data, bytes);
@@ -562,16 +567,17 @@ tr71:
 		}
 	goto st197;
 tr91:
-#line 248 "src/memcached-grammar.rl"
+#line 247 "src/memcached-grammar.rl"
 	{cas = natoq(fstart, p);}
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -596,7 +602,7 @@ tr91:
 		}
 #line 86 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL || expired(tuple))
 				obuf_dup(out, "NOT_FOUND\r\n", 11);
@@ -609,12 +615,13 @@ tr91:
 tr95:
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -639,7 +646,7 @@ tr95:
 		}
 #line 86 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL || expired(tuple))
 				obuf_dup(out, "NOT_FOUND\r\n", 11);
@@ -654,12 +661,13 @@ tr105:
 	{ noreply = true; }
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -684,7 +692,7 @@ tr105:
 		}
 #line 86 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL || expired(tuple))
 				obuf_dup(out, "NOT_FOUND\r\n", 11);
@@ -695,7 +703,7 @@ tr105:
 		}
 	goto st197;
 tr118:
-#line 249 "src/memcached-grammar.rl"
+#line 248 "src/memcached-grammar.rl"
 	{incr = natoq(fstart, p);}
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
@@ -709,11 +717,11 @@ tr118:
 	{
 			struct meta *m;
 			struct tbuf *b;
-			void *field;
+			const void *field;
 			u32 value_len;
 			u64 value;
 
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL || expired(tuple)) {
 				obuf_dup(out, "NOT_FOUND\r\n", 11);
@@ -737,7 +745,7 @@ tr118:
 					exptime = m->exptime;
 					flags = m->flags;
 
-					b = tbuf_alloc(fiber->gc_pool);
+					b = tbuf_new(fiber->gc_pool);
 					tbuf_printf(b, "%"PRIu64, value);
 					data = b->data;
 					bytes = b->size;
@@ -774,11 +782,11 @@ tr122:
 	{
 			struct meta *m;
 			struct tbuf *b;
-			void *field;
+			const void *field;
 			u32 value_len;
 			u64 value;
 
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL || expired(tuple)) {
 				obuf_dup(out, "NOT_FOUND\r\n", 11);
@@ -802,7 +810,7 @@ tr122:
 					exptime = m->exptime;
 					flags = m->flags;
 
-					b = tbuf_alloc(fiber->gc_pool);
+					b = tbuf_new(fiber->gc_pool);
 					tbuf_printf(b, "%"PRIu64, value);
 					data = b->data;
 					bytes = b->size;
@@ -841,11 +849,11 @@ tr132:
 	{
 			struct meta *m;
 			struct tbuf *b;
-			void *field;
+			const void *field;
 			u32 value_len;
 			u64 value;
 
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL || expired(tuple)) {
 				obuf_dup(out, "NOT_FOUND\r\n", 11);
@@ -869,7 +877,7 @@ tr132:
 					exptime = m->exptime;
 					flags = m->flags;
 
-					b = tbuf_alloc(fiber->gc_pool);
+					b = tbuf_new(fiber->gc_pool);
 					tbuf_printf(b, "%"PRIu64, value);
 					data = b->data;
 					bytes = b->size;
@@ -904,7 +912,7 @@ tr141:
 		}
 #line 179 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL || expired(tuple)) {
 				obuf_dup(out, "NOT_FOUND\r\n", 11);
@@ -922,7 +930,7 @@ tr141:
 		}
 	goto st197;
 tr146:
-#line 239 "src/memcached-grammar.rl"
+#line 238 "src/memcached-grammar.rl"
 	{
 			exptime = natoq(fstart, p);
 			if (exptime > 0 && exptime <= 60*60*24*30)
@@ -938,7 +946,7 @@ tr146:
 		}
 #line 179 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL || expired(tuple)) {
 				obuf_dup(out, "NOT_FOUND\r\n", 11);
@@ -968,7 +976,7 @@ tr157:
 		}
 #line 179 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL || expired(tuple)) {
 				obuf_dup(out, "NOT_FOUND\r\n", 11);
@@ -996,13 +1004,13 @@ tr169:
 		}
 #line 208 "src/memcached-grammar.rl"
 	{
-			struct fiber *f = fiber_create("flush_all", flush_all);
+			struct fiber *f = fiber_new("flush_all", flush_all);
 			fiber_call(f, flush_delay);
 			obuf_dup(out, "OK\r\n", 4);
 		}
 	goto st197;
 tr174:
-#line 250 "src/memcached-grammar.rl"
+#line 249 "src/memcached-grammar.rl"
 	{flush_delay = natoq(fstart, p);}
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
@@ -1014,7 +1022,7 @@ tr174:
 		}
 #line 208 "src/memcached-grammar.rl"
 	{
-			struct fiber *f = fiber_create("flush_all", flush_all);
+			struct fiber *f = fiber_new("flush_all", flush_all);
 			fiber_call(f, flush_delay);
 			obuf_dup(out, "OK\r\n", 4);
 		}
@@ -1032,7 +1040,7 @@ tr185:
 		}
 #line 208 "src/memcached-grammar.rl"
 	{
-			struct fiber *f = fiber_create("flush_all", flush_all);
+			struct fiber *f = fiber_new("flush_all", flush_all);
 			fiber_call(f, flush_delay);
 			obuf_dup(out, "OK\r\n", 4);
 		}
@@ -1073,16 +1081,17 @@ tr213:
 		}
 	goto st197;
 tr233:
-#line 247 "src/memcached-grammar.rl"
+#line 246 "src/memcached-grammar.rl"
 	{bytes = natoq(fstart, p);}
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -1107,7 +1116,7 @@ tr233:
 		}
 #line 77 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL || expired(tuple))
 				obuf_dup(out, "NOT_STORED\r\n", 12);
@@ -1118,12 +1127,13 @@ tr233:
 tr237:
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -1148,7 +1158,7 @@ tr237:
 		}
 #line 77 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL || expired(tuple))
 				obuf_dup(out, "NOT_STORED\r\n", 12);
@@ -1161,12 +1171,13 @@ tr246:
 	{ noreply = true; }
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -1191,7 +1202,7 @@ tr246:
 		}
 #line 77 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			struct tuple *tuple = find(key);
 			if (tuple == NULL || expired(tuple))
 				obuf_dup(out, "NOT_STORED\r\n", 12);
@@ -1200,16 +1211,17 @@ tr246:
 		}
 	goto st197;
 tr263:
-#line 247 "src/memcached-grammar.rl"
+#line 246 "src/memcached-grammar.rl"
 	{bytes = natoq(fstart, p);}
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -1234,19 +1246,20 @@ tr263:
 		}
 #line 63 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			STORE;
 		}
 	goto st197;
 tr267:
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -1271,7 +1284,7 @@ tr267:
 		}
 #line 63 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			STORE;
 		}
 	goto st197;
@@ -1280,12 +1293,13 @@ tr276:
 	{ noreply = true; }
 #line 280 "src/memcached-grammar.rl"
 	{ p++; }
-#line 252 "src/memcached-grammar.rl"
+#line 251 "src/memcached-grammar.rl"
 	{
 			size_t parsed = p - in->pos;
 			while (ibuf_size(in) - parsed < bytes + 2) {
-				if (coio_bread(coio, in, bytes + 2 - (pe - p)) <= 0)
-					return -1;
+				size_t to_read = bytes + 2 - (pe - p);
+				if (coio_bread(coio, in, to_read) < to_read)
+					return -1; /* premature EOF */
 			}
 			/*
 			 * Buffered read may have reallocated the
@@ -1310,7 +1324,7 @@ tr276:
 		}
 #line 63 "src/memcached-grammar.rl"
 	{
-			key = read_field(keys);
+			key = tbuf_read_field(keys);
 			STORE;
 		}
 	goto st197;
@@ -1332,10 +1346,10 @@ st197:
 	if ( ++p == pe )
 		goto _test_eof197;
 case 197:
-#line 1336 "src/memcached-grammar.m"
+#line 1350 "src/memcached-grammar.m"
 	goto st0;
 tr27:
-#line 247 "src/memcached-grammar.rl"
+#line 246 "src/memcached-grammar.rl"
 	{bytes = natoq(fstart, p);}
 	goto st13;
 tr40:
@@ -1346,19 +1360,19 @@ st13:
 	if ( ++p == pe )
 		goto _test_eof13;
 case 13:
-#line 1350 "src/memcached-grammar.m"
+#line 1364 "src/memcached-grammar.m"
 	if ( (*p) == 10 )
 		goto tr30;
 	goto st0;
 tr28:
-#line 247 "src/memcached-grammar.rl"
+#line 246 "src/memcached-grammar.rl"
 	{bytes = natoq(fstart, p);}
 	goto st14;
 st14:
 	if ( ++p == pe )
 		goto _test_eof14;
 case 14:
-#line 1362 "src/memcached-grammar.m"
+#line 1376 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 32: goto st14;
 		case 78: goto st15;
@@ -1483,7 +1497,7 @@ st27:
 	if ( ++p == pe )
 		goto _test_eof27;
 case 27:
-#line 1487 "src/memcached-grammar.m"
+#line 1501 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 13: goto st0;
 		case 32: goto st27;
@@ -1497,8 +1511,7 @@ tr46:
 			fstart = p;
 			for (; p < pe && *p != ' ' && *p != '\r' && *p != '\n'; p++);
 			if ( *p == ' ' || *p == '\r' || *p == '\n') {
-				write_varint32(keys, p - fstart);
-				tbuf_append(keys, fstart, p - fstart);
+				tbuf_store_field(keys, fstart, p - fstart);
 				keys_count++;
 				p--;
 			} else
@@ -1509,7 +1522,7 @@ st28:
 	if ( ++p == pe )
 		goto _test_eof28;
 case 28:
-#line 1513 "src/memcached-grammar.m"
+#line 1526 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st29;
 	goto st0;
@@ -1530,21 +1543,21 @@ st30:
 	if ( ++p == pe )
 		goto _test_eof30;
 case 30:
-#line 1534 "src/memcached-grammar.m"
+#line 1547 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto tr50;
 	if ( 48 <= (*p) && (*p) <= 57 )
 		goto st30;
 	goto st0;
 tr50:
-#line 246 "src/memcached-grammar.rl"
+#line 245 "src/memcached-grammar.rl"
 	{flags = natoq(fstart, p);}
 	goto st31;
 st31:
 	if ( ++p == pe )
 		goto _test_eof31;
 case 31:
-#line 1548 "src/memcached-grammar.m"
+#line 1561 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st31;
 	if ( 48 <= (*p) && (*p) <= 57 )
@@ -1558,14 +1571,14 @@ st32:
 	if ( ++p == pe )
 		goto _test_eof32;
 case 32:
-#line 1562 "src/memcached-grammar.m"
+#line 1575 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto tr54;
 	if ( 48 <= (*p) && (*p) <= 57 )
 		goto st32;
 	goto st0;
 tr54:
-#line 239 "src/memcached-grammar.rl"
+#line 238 "src/memcached-grammar.rl"
 	{
 			exptime = natoq(fstart, p);
 			if (exptime > 0 && exptime <= 60*60*24*30)
@@ -1576,7 +1589,7 @@ st33:
 	if ( ++p == pe )
 		goto _test_eof33;
 case 33:
-#line 1580 "src/memcached-grammar.m"
+#line 1593 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st33;
 	if ( 48 <= (*p) && (*p) <= 57 )
@@ -1590,7 +1603,7 @@ st34:
 	if ( ++p == pe )
 		goto _test_eof34;
 case 34:
-#line 1594 "src/memcached-grammar.m"
+#line 1607 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr58;
 		case 13: goto tr59;
@@ -1600,7 +1613,7 @@ case 34:
 		goto st34;
 	goto st0;
 tr59:
-#line 247 "src/memcached-grammar.rl"
+#line 246 "src/memcached-grammar.rl"
 	{bytes = natoq(fstart, p);}
 	goto st35;
 tr72:
@@ -1611,19 +1624,19 @@ st35:
 	if ( ++p == pe )
 		goto _test_eof35;
 case 35:
-#line 1615 "src/memcached-grammar.m"
+#line 1628 "src/memcached-grammar.m"
 	if ( (*p) == 10 )
 		goto tr62;
 	goto st0;
 tr60:
-#line 247 "src/memcached-grammar.rl"
+#line 246 "src/memcached-grammar.rl"
 	{bytes = natoq(fstart, p);}
 	goto st36;
 st36:
 	if ( ++p == pe )
 		goto _test_eof36;
 case 36:
-#line 1627 "src/memcached-grammar.m"
+#line 1640 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 32: goto st36;
 		case 78: goto st37;
@@ -1735,8 +1748,7 @@ tr76:
 			fstart = p;
 			for (; p < pe && *p != ' ' && *p != '\r' && *p != '\n'; p++);
 			if ( *p == ' ' || *p == '\r' || *p == '\n') {
-				write_varint32(keys, p - fstart);
-				tbuf_append(keys, fstart, p - fstart);
+				tbuf_store_field(keys, fstart, p - fstart);
 				keys_count++;
 				p--;
 			} else
@@ -1747,7 +1759,7 @@ st48:
 	if ( ++p == pe )
 		goto _test_eof48;
 case 48:
-#line 1751 "src/memcached-grammar.m"
+#line 1763 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st49;
 	goto st0;
@@ -1768,21 +1780,21 @@ st50:
 	if ( ++p == pe )
 		goto _test_eof50;
 case 50:
-#line 1772 "src/memcached-grammar.m"
+#line 1784 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto tr79;
 	if ( 48 <= (*p) && (*p) <= 57 )
 		goto st50;
 	goto st0;
 tr79:
-#line 246 "src/memcached-grammar.rl"
+#line 245 "src/memcached-grammar.rl"
 	{flags = natoq(fstart, p);}
 	goto st51;
 st51:
 	if ( ++p == pe )
 		goto _test_eof51;
 case 51:
-#line 1786 "src/memcached-grammar.m"
+#line 1798 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st51;
 	if ( 48 <= (*p) && (*p) <= 57 )
@@ -1796,14 +1808,14 @@ st52:
 	if ( ++p == pe )
 		goto _test_eof52;
 case 52:
-#line 1800 "src/memcached-grammar.m"
+#line 1812 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto tr83;
 	if ( 48 <= (*p) && (*p) <= 57 )
 		goto st52;
 	goto st0;
 tr83:
-#line 239 "src/memcached-grammar.rl"
+#line 238 "src/memcached-grammar.rl"
 	{
 			exptime = natoq(fstart, p);
 			if (exptime > 0 && exptime <= 60*60*24*30)
@@ -1814,7 +1826,7 @@ st53:
 	if ( ++p == pe )
 		goto _test_eof53;
 case 53:
-#line 1818 "src/memcached-grammar.m"
+#line 1830 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st53;
 	if ( 48 <= (*p) && (*p) <= 57 )
@@ -1828,21 +1840,21 @@ st54:
 	if ( ++p == pe )
 		goto _test_eof54;
 case 54:
-#line 1832 "src/memcached-grammar.m"
+#line 1844 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto tr87;
 	if ( 48 <= (*p) && (*p) <= 57 )
 		goto st54;
 	goto st0;
 tr87:
-#line 247 "src/memcached-grammar.rl"
+#line 246 "src/memcached-grammar.rl"
 	{bytes = natoq(fstart, p);}
 	goto st55;
 st55:
 	if ( ++p == pe )
 		goto _test_eof55;
 case 55:
-#line 1846 "src/memcached-grammar.m"
+#line 1858 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st55;
 	if ( 48 <= (*p) && (*p) <= 57 )
@@ -1856,7 +1868,7 @@ st56:
 	if ( ++p == pe )
 		goto _test_eof56;
 case 56:
-#line 1860 "src/memcached-grammar.m"
+#line 1872 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr91;
 		case 13: goto tr92;
@@ -1870,26 +1882,26 @@ tr106:
 	{ noreply = true; }
 	goto st57;
 tr92:
-#line 248 "src/memcached-grammar.rl"
+#line 247 "src/memcached-grammar.rl"
 	{cas = natoq(fstart, p);}
 	goto st57;
 st57:
 	if ( ++p == pe )
 		goto _test_eof57;
 case 57:
-#line 1881 "src/memcached-grammar.m"
+#line 1893 "src/memcached-grammar.m"
 	if ( (*p) == 10 )
 		goto tr95;
 	goto st0;
 tr93:
-#line 248 "src/memcached-grammar.rl"
+#line 247 "src/memcached-grammar.rl"
 	{cas = natoq(fstart, p);}
 	goto st58;
 st58:
 	if ( ++p == pe )
 		goto _test_eof58;
 case 58:
-#line 1893 "src/memcached-grammar.m"
+#line 1905 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr95;
 		case 13: goto st57;
@@ -1970,7 +1982,7 @@ st66:
 	if ( ++p == pe )
 		goto _test_eof66;
 case 66:
-#line 1974 "src/memcached-grammar.m"
+#line 1986 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr95;
 		case 13: goto st57;
@@ -2025,7 +2037,7 @@ st71:
 	if ( ++p == pe )
 		goto _test_eof71;
 case 71:
-#line 2029 "src/memcached-grammar.m"
+#line 2041 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 13: goto st0;
 		case 32: goto st71;
@@ -2039,8 +2051,7 @@ tr114:
 			fstart = p;
 			for (; p < pe && *p != ' ' && *p != '\r' && *p != '\n'; p++);
 			if ( *p == ' ' || *p == '\r' || *p == '\n') {
-				write_varint32(keys, p - fstart);
-				tbuf_append(keys, fstart, p - fstart);
+				tbuf_store_field(keys, fstart, p - fstart);
 				keys_count++;
 				p--;
 			} else
@@ -2051,7 +2062,7 @@ st72:
 	if ( ++p == pe )
 		goto _test_eof72;
 case 72:
-#line 2055 "src/memcached-grammar.m"
+#line 2066 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st73;
 	goto st0;
@@ -2072,7 +2083,7 @@ st74:
 	if ( ++p == pe )
 		goto _test_eof74;
 case 74:
-#line 2076 "src/memcached-grammar.m"
+#line 2087 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr118;
 		case 13: goto tr119;
@@ -2086,26 +2097,26 @@ tr133:
 	{ noreply = true; }
 	goto st75;
 tr119:
-#line 249 "src/memcached-grammar.rl"
+#line 248 "src/memcached-grammar.rl"
 	{incr = natoq(fstart, p);}
 	goto st75;
 st75:
 	if ( ++p == pe )
 		goto _test_eof75;
 case 75:
-#line 2097 "src/memcached-grammar.m"
+#line 2108 "src/memcached-grammar.m"
 	if ( (*p) == 10 )
 		goto tr122;
 	goto st0;
 tr120:
-#line 249 "src/memcached-grammar.rl"
+#line 248 "src/memcached-grammar.rl"
 	{incr = natoq(fstart, p);}
 	goto st76;
 st76:
 	if ( ++p == pe )
 		goto _test_eof76;
 case 76:
-#line 2109 "src/memcached-grammar.m"
+#line 2120 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr122;
 		case 13: goto st75;
@@ -2186,7 +2197,7 @@ st84:
 	if ( ++p == pe )
 		goto _test_eof84;
 case 84:
-#line 2190 "src/memcached-grammar.m"
+#line 2201 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr122;
 		case 13: goto st75;
@@ -2244,8 +2255,7 @@ tr140:
 			fstart = p;
 			for (; p < pe && *p != ' ' && *p != '\r' && *p != '\n'; p++);
 			if ( *p == ' ' || *p == '\r' || *p == '\n') {
-				write_varint32(keys, p - fstart);
-				tbuf_append(keys, fstart, p - fstart);
+				tbuf_store_field(keys, fstart, p - fstart);
 				keys_count++;
 				p--;
 			} else
@@ -2256,7 +2266,7 @@ st90:
 	if ( ++p == pe )
 		goto _test_eof90;
 case 90:
-#line 2260 "src/memcached-grammar.m"
+#line 2270 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr141;
 		case 13: goto st91;
@@ -2264,7 +2274,7 @@ case 90:
 	}
 	goto st0;
 tr147:
-#line 239 "src/memcached-grammar.rl"
+#line 238 "src/memcached-grammar.rl"
 	{
 			exptime = natoq(fstart, p);
 			if (exptime > 0 && exptime <= 60*60*24*30)
@@ -2279,7 +2289,7 @@ st91:
 	if ( ++p == pe )
 		goto _test_eof91;
 case 91:
-#line 2283 "src/memcached-grammar.m"
+#line 2293 "src/memcached-grammar.m"
 	if ( (*p) == 10 )
 		goto tr141;
 	goto st0;
@@ -2305,7 +2315,7 @@ st93:
 	if ( ++p == pe )
 		goto _test_eof93;
 case 93:
-#line 2309 "src/memcached-grammar.m"
+#line 2319 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr146;
 		case 13: goto tr147;
@@ -2315,7 +2325,7 @@ case 93:
 		goto st93;
 	goto st0;
 tr148:
-#line 239 "src/memcached-grammar.rl"
+#line 238 "src/memcached-grammar.rl"
 	{
 			exptime = natoq(fstart, p);
 			if (exptime > 0 && exptime <= 60*60*24*30)
@@ -2326,7 +2336,7 @@ st94:
 	if ( ++p == pe )
 		goto _test_eof94;
 case 94:
-#line 2330 "src/memcached-grammar.m"
+#line 2340 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr141;
 		case 13: goto st91;
@@ -2407,7 +2417,7 @@ st102:
 	if ( ++p == pe )
 		goto _test_eof102;
 case 102:
-#line 2411 "src/memcached-grammar.m"
+#line 2421 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr141;
 		case 13: goto st91;
@@ -2499,14 +2509,14 @@ tr186:
 	{ noreply = true; }
 	goto st112;
 tr175:
-#line 250 "src/memcached-grammar.rl"
+#line 249 "src/memcached-grammar.rl"
 	{flush_delay = natoq(fstart, p);}
 	goto st112;
 st112:
 	if ( ++p == pe )
 		goto _test_eof112;
 case 112:
-#line 2510 "src/memcached-grammar.m"
+#line 2520 "src/memcached-grammar.m"
 	if ( (*p) == 10 )
 		goto tr169;
 	goto st0;
@@ -2532,7 +2542,7 @@ st114:
 	if ( ++p == pe )
 		goto _test_eof114;
 case 114:
-#line 2536 "src/memcached-grammar.m"
+#line 2546 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr174;
 		case 13: goto tr175;
@@ -2542,14 +2552,14 @@ case 114:
 		goto st114;
 	goto st0;
 tr176:
-#line 250 "src/memcached-grammar.rl"
+#line 249 "src/memcached-grammar.rl"
 	{flush_delay = natoq(fstart, p);}
 	goto st115;
 st115:
 	if ( ++p == pe )
 		goto _test_eof115;
 case 115:
-#line 2553 "src/memcached-grammar.m"
+#line 2563 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr169;
 		case 13: goto st112;
@@ -2630,7 +2640,7 @@ st123:
 	if ( ++p == pe )
 		goto _test_eof123;
 case 123:
-#line 2634 "src/memcached-grammar.m"
+#line 2644 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr169;
 		case 13: goto st112;
@@ -2677,7 +2687,7 @@ st127:
 	if ( ++p == pe )
 		goto _test_eof127;
 case 127:
-#line 2681 "src/memcached-grammar.m"
+#line 2691 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 13: goto st0;
 		case 32: goto st127;
@@ -2691,8 +2701,7 @@ tr193:
 			fstart = p;
 			for (; p < pe && *p != ' ' && *p != '\r' && *p != '\n'; p++);
 			if ( *p == ' ' || *p == '\r' || *p == '\n') {
-				write_varint32(keys, p - fstart);
-				tbuf_append(keys, fstart, p - fstart);
+				tbuf_store_field(keys, fstart, p - fstart);
 				keys_count++;
 				p--;
 			} else
@@ -2703,7 +2712,7 @@ st128:
 	if ( ++p == pe )
 		goto _test_eof128;
 case 128:
-#line 2707 "src/memcached-grammar.m"
+#line 2716 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr195;
 		case 13: goto st129;
@@ -2951,8 +2960,7 @@ tr222:
 			fstart = p;
 			for (; p < pe && *p != ' ' && *p != '\r' && *p != '\n'; p++);
 			if ( *p == ' ' || *p == '\r' || *p == '\n') {
-				write_varint32(keys, p - fstart);
-				tbuf_append(keys, fstart, p - fstart);
+				tbuf_store_field(keys, fstart, p - fstart);
 				keys_count++;
 				p--;
 			} else
@@ -2963,7 +2971,7 @@ st156:
 	if ( ++p == pe )
 		goto _test_eof156;
 case 156:
-#line 2967 "src/memcached-grammar.m"
+#line 2975 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st157;
 	goto st0;
@@ -2984,21 +2992,21 @@ st158:
 	if ( ++p == pe )
 		goto _test_eof158;
 case 158:
-#line 2988 "src/memcached-grammar.m"
+#line 2996 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto tr225;
 	if ( 48 <= (*p) && (*p) <= 57 )
 		goto st158;
 	goto st0;
 tr225:
-#line 246 "src/memcached-grammar.rl"
+#line 245 "src/memcached-grammar.rl"
 	{flags = natoq(fstart, p);}
 	goto st159;
 st159:
 	if ( ++p == pe )
 		goto _test_eof159;
 case 159:
-#line 3002 "src/memcached-grammar.m"
+#line 3010 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st159;
 	if ( 48 <= (*p) && (*p) <= 57 )
@@ -3012,14 +3020,14 @@ st160:
 	if ( ++p == pe )
 		goto _test_eof160;
 case 160:
-#line 3016 "src/memcached-grammar.m"
+#line 3024 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto tr229;
 	if ( 48 <= (*p) && (*p) <= 57 )
 		goto st160;
 	goto st0;
 tr229:
-#line 239 "src/memcached-grammar.rl"
+#line 238 "src/memcached-grammar.rl"
 	{
 			exptime = natoq(fstart, p);
 			if (exptime > 0 && exptime <= 60*60*24*30)
@@ -3030,7 +3038,7 @@ st161:
 	if ( ++p == pe )
 		goto _test_eof161;
 case 161:
-#line 3034 "src/memcached-grammar.m"
+#line 3042 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st161;
 	if ( 48 <= (*p) && (*p) <= 57 )
@@ -3044,7 +3052,7 @@ st162:
 	if ( ++p == pe )
 		goto _test_eof162;
 case 162:
-#line 3048 "src/memcached-grammar.m"
+#line 3056 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr233;
 		case 13: goto tr234;
@@ -3054,7 +3062,7 @@ case 162:
 		goto st162;
 	goto st0;
 tr234:
-#line 247 "src/memcached-grammar.rl"
+#line 246 "src/memcached-grammar.rl"
 	{bytes = natoq(fstart, p);}
 	goto st163;
 tr247:
@@ -3065,19 +3073,19 @@ st163:
 	if ( ++p == pe )
 		goto _test_eof163;
 case 163:
-#line 3069 "src/memcached-grammar.m"
+#line 3077 "src/memcached-grammar.m"
 	if ( (*p) == 10 )
 		goto tr237;
 	goto st0;
 tr235:
-#line 247 "src/memcached-grammar.rl"
+#line 246 "src/memcached-grammar.rl"
 	{bytes = natoq(fstart, p);}
 	goto st164;
 st164:
 	if ( ++p == pe )
 		goto _test_eof164;
 case 164:
-#line 3081 "src/memcached-grammar.m"
+#line 3089 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 32: goto st164;
 		case 78: goto st165;
@@ -3191,8 +3199,7 @@ tr252:
 			fstart = p;
 			for (; p < pe && *p != ' ' && *p != '\r' && *p != '\n'; p++);
 			if ( *p == ' ' || *p == '\r' || *p == '\n') {
-				write_varint32(keys, p - fstart);
-				tbuf_append(keys, fstart, p - fstart);
+				tbuf_store_field(keys, fstart, p - fstart);
 				keys_count++;
 				p--;
 			} else
@@ -3203,7 +3210,7 @@ st176:
 	if ( ++p == pe )
 		goto _test_eof176;
 case 176:
-#line 3207 "src/memcached-grammar.m"
+#line 3214 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st177;
 	goto st0;
@@ -3224,21 +3231,21 @@ st178:
 	if ( ++p == pe )
 		goto _test_eof178;
 case 178:
-#line 3228 "src/memcached-grammar.m"
+#line 3235 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto tr255;
 	if ( 48 <= (*p) && (*p) <= 57 )
 		goto st178;
 	goto st0;
 tr255:
-#line 246 "src/memcached-grammar.rl"
+#line 245 "src/memcached-grammar.rl"
 	{flags = natoq(fstart, p);}
 	goto st179;
 st179:
 	if ( ++p == pe )
 		goto _test_eof179;
 case 179:
-#line 3242 "src/memcached-grammar.m"
+#line 3249 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st179;
 	if ( 48 <= (*p) && (*p) <= 57 )
@@ -3252,14 +3259,14 @@ st180:
 	if ( ++p == pe )
 		goto _test_eof180;
 case 180:
-#line 3256 "src/memcached-grammar.m"
+#line 3263 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto tr259;
 	if ( 48 <= (*p) && (*p) <= 57 )
 		goto st180;
 	goto st0;
 tr259:
-#line 239 "src/memcached-grammar.rl"
+#line 238 "src/memcached-grammar.rl"
 	{
 			exptime = natoq(fstart, p);
 			if (exptime > 0 && exptime <= 60*60*24*30)
@@ -3270,7 +3277,7 @@ st181:
 	if ( ++p == pe )
 		goto _test_eof181;
 case 181:
-#line 3274 "src/memcached-grammar.m"
+#line 3281 "src/memcached-grammar.m"
 	if ( (*p) == 32 )
 		goto st181;
 	if ( 48 <= (*p) && (*p) <= 57 )
@@ -3284,7 +3291,7 @@ st182:
 	if ( ++p == pe )
 		goto _test_eof182;
 case 182:
-#line 3288 "src/memcached-grammar.m"
+#line 3295 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 10: goto tr263;
 		case 13: goto tr264;
@@ -3294,7 +3301,7 @@ case 182:
 		goto st182;
 	goto st0;
 tr264:
-#line 247 "src/memcached-grammar.rl"
+#line 246 "src/memcached-grammar.rl"
 	{bytes = natoq(fstart, p);}
 	goto st183;
 tr277:
@@ -3305,19 +3312,19 @@ st183:
 	if ( ++p == pe )
 		goto _test_eof183;
 case 183:
-#line 3309 "src/memcached-grammar.m"
+#line 3316 "src/memcached-grammar.m"
 	if ( (*p) == 10 )
 		goto tr267;
 	goto st0;
 tr265:
-#line 247 "src/memcached-grammar.rl"
+#line 246 "src/memcached-grammar.rl"
 	{bytes = natoq(fstart, p);}
 	goto st184;
 st184:
 	if ( ++p == pe )
 		goto _test_eof184;
 case 184:
-#line 3321 "src/memcached-grammar.m"
+#line 3328 "src/memcached-grammar.m"
 	switch( (*p) ) {
 		case 32: goto st184;
 		case 78: goto st185;
