@@ -782,6 +782,13 @@ wal_schedule(ev_watcher *watcher, int event __attribute__((unused)))
 	(void) tt_pthread_mutex_unlock(&writer->mutex);
 
 	wal_schedule_queue(&commit);
+	/*
+	 * Perform a cascading abort of all transactions which
+	 * depend on the transaction which failed to get written
+	 * to the write ahead log. Abort transactions
+	 * in reverse order, performing a playback of the
+	 * in-memory database state.
+	 */
 	STAILQ_REVERSE(&rollback, wal_write_request, wal_fifo_entry);
 	wal_schedule_queue(&rollback);
 }
@@ -996,7 +1003,7 @@ wal_fill_batch(struct log_io *wal, struct fio_batch *batch, int rows_per_wal,
 	       struct wal_write_request *req)
 {
 	int max_rows = wal->is_inprogress ? 1 : rows_per_wal - wal->rows;
-	/* Post-condition of successful by wal_opt_rotate(). */
+	/* Post-condition of successful wal_opt_rotate(). */
 	assert(max_rows > 0);
 	fio_batch_start(batch, max_rows);
 	while (req != NULL && ! fio_batch_is_full(batch)) {
@@ -1092,7 +1099,7 @@ wal_writer_thread(void *worker_args)
  */
 int
 wal_write(struct recovery_state *r, i64 lsn, u64 cookie,
-	  u16 op, struct tbuf *row)
+	  u16 op, const void *row, u32 row_len)
 {
 	say_debug("wal_write lsn=%" PRIi64, lsn);
 	ERROR_INJECT_RETURN(ERRINJ_WAL_IO);
@@ -1104,12 +1111,12 @@ wal_write(struct recovery_state *r, i64 lsn, u64 cookie,
 
 	struct wal_write_request *req =
 		palloc(fiber->gc_pool, sizeof(struct wal_write_request) +
-		       sizeof(op) + row->size);
+		       sizeof(op) + row_len);
 
 	req->fiber = fiber;
 	req->res = -1;
 	row_v11_fill(&req->row, lsn, XLOG, cookie, &op, sizeof(op),
-		     row->data, row->size);
+		     row, row_len);
 
 	(void) tt_pthread_mutex_lock(&writer->mutex);
 
@@ -1250,6 +1257,9 @@ read_log(const char *filename,
 
 	FILE *f = fopen(filename, "r");
 	struct log_io *l = log_io_open(dir, LOG_READ, filename, NONE, f);
+	if (l == NULL)
+		return -1;
+
 	struct log_io_cursor i;
 
 	log_io_cursor_open(&i, l);
