@@ -106,7 +106,7 @@ pull_from_remote(va_list ap)
 			fiber_setcancellable(true);
 			if (! evio_is_active(&coio)) {
 				if (iobuf == NULL)
-					iobuf = iobuf_new(fiber->name);
+					iobuf = iobuf_new(fiber_name(fiber));
 				remote_connect(&coio, &r->remote->addr,
 					       r->confirmed_lsn + 1, &err);
 				warning_said = false;
