@@ -296,6 +296,20 @@ static void tt_tnt_iter3(struct tt_test *test) {
 	tnt_list_free(l);
 }
 
+/* iterator empty tuple */
+static void tt_tnt_iter4(struct tt_test *test) {
+	char buf[4];
+	memset(buf, 0, sizeof(buf));
+	struct tnt_tuple t;
+	tnt_tuple_init(&t);
+	tnt_tuple_set(&t, buf, sizeof(buf));
+	struct tnt_iter i;
+	tnt_iter(&i, &t);
+	TT_ASSERT(tnt_next(&i) == 0);
+	tnt_iter_free(&i);
+	tnt_tuple_free(&t);
+}
+
 /* marshal ping */
 static void tt_tnt_marshal_ping(struct tt_test *test) {
 	struct tnt_stream s;
@@ -781,11 +795,13 @@ static void tt_tnt_net_reply(struct tt_test *test) {
 	net.wrcnt -= 2;
 }
 
+extern struct tnt_lex_keyword tnt_sql_keywords[];
+
 /* lex ws */
 static void tt_tnt_lex_ws(struct tt_test *test) {
-	unsigned char sz[] = " 	# abcde fghjk ## hh\n   # zzz\n";
+	unsigned char sz[] = " 	# abcde fghjk ## hh\n   # zzz\n#NOCR";
 	struct tnt_lex l;
-	tnt_lex_init(&l, sz, sizeof(sz) - 1);
+	tnt_lex_init(&l, tnt_sql_keywords, sz, sizeof(sz) - 1);
 	struct tnt_tk *tk;
 	TT_ASSERT(tnt_lex(&l, &tk) == TNT_TK_EOF);
 	tnt_lex_free(&l);
@@ -796,7 +812,7 @@ static void tt_tnt_lex_int(struct tt_test *test) {
 	unsigned char sz[] = "\f\r\n 123 34\n\t\r56 888L56 2147483646 2147483647 "
 		             "-2147483648 -2147483649 72057594037927935";
 	struct tnt_lex l;
-	tnt_lex_init(&l, sz, sizeof(sz) - 1);
+	tnt_lex_init(&l, tnt_sql_keywords, sz, sizeof(sz) - 1);
 	struct tnt_tk *tk;
 	TT_ASSERT(tnt_lex(&l, &tk) == TNT_TK_NUM32 && TNT_TK_I32(tk) == 123);
 	TT_ASSERT(tnt_lex(&l, &tk) == TNT_TK_NUM32 && TNT_TK_I32(tk) == 34);
@@ -817,7 +833,7 @@ static void tt_tnt_lex_int(struct tt_test *test) {
 static void tt_tnt_lex_punct(struct tt_test *test) {
 	unsigned char sz[] = "123,34\n-10\t:\r(56)";
 	struct tnt_lex l;
-	tnt_lex_init(&l, sz, sizeof(sz) - 1);
+	tnt_lex_init(&l, tnt_sql_keywords, sz, sizeof(sz) - 1);
 	struct tnt_tk *tk;
 	TT_ASSERT(tnt_lex(&l, &tk) == TNT_TK_NUM32 && TNT_TK_I32(tk) == 123);
 	TT_ASSERT(tnt_lex(&l, &tk) == ',' && TNT_TK_I32(tk) == ',');
@@ -835,7 +851,7 @@ static void tt_tnt_lex_punct(struct tt_test *test) {
 static void tt_tnt_lex_str(struct tt_test *test) {
 	unsigned char sz[] = "  'hello'\n\t  'world'  'всем привет!'";
 	struct tnt_lex l;
-	tnt_lex_init(&l, sz, sizeof(sz) - 1);
+	tnt_lex_init(&l, tnt_sql_keywords, sz, sizeof(sz) - 1);
 	struct tnt_tk *tk;
 	TT_ASSERT(tnt_lex(&l, &tk) == TNT_TK_STRING &&
 	       TNT_TK_S(tk)->size == 5 &&
@@ -854,7 +870,7 @@ static void tt_tnt_lex_str(struct tt_test *test) {
 static void tt_tnt_lex_ids(struct tt_test *test) {
 	unsigned char sz[] = "  hello\nэтот безумный безумный мир\t  world  ";
 	struct tnt_lex l;
-	tnt_lex_init(&l, sz, sizeof(sz) - 1);
+	tnt_lex_init(&l, tnt_sql_keywords, sz, sizeof(sz) - 1);
 	struct tnt_tk *tk;
 	TT_ASSERT(tnt_lex(&l, &tk) == TNT_TK_ID &&
 	       TNT_TK_S(tk)->size == 5 &&
@@ -882,7 +898,7 @@ static void tt_tnt_lex_ids(struct tt_test *test) {
 static void tt_tnt_lex_kt(struct tt_test *test) {
 	unsigned char sz[] = "  k0\n\tk20 t0 k1000 t55 k001 t8";
 	struct tnt_lex l;
-	tnt_lex_init(&l, sz, sizeof(sz) - 1);
+	tnt_lex_init(&l, tnt_sql_keywords, sz, sizeof(sz) - 1);
 	struct tnt_tk *tk;
 	TT_ASSERT(tnt_lex(&l, &tk) == TNT_TK_KEY && TNT_TK_I32(tk) == 0);
 	TT_ASSERT(tnt_lex(&l, &tk) == TNT_TK_KEY && TNT_TK_I32(tk) == 20);
@@ -899,7 +915,7 @@ static void tt_tnt_lex_kt(struct tt_test *test) {
 static void tt_tnt_lex_kw(struct tt_test *test) {
 	unsigned char sz[] = "  INSERT UPDATE INTO OR FROM WHERE VALUES";
 	struct tnt_lex l;
-	tnt_lex_init(&l, sz, sizeof(sz) - 1);
+	tnt_lex_init(&l, tnt_sql_keywords, sz, sizeof(sz) - 1);
 	struct tnt_tk *tk;
 	TT_ASSERT(tnt_lex(&l, &tk) == TNT_TK_INSERT);
 	TT_ASSERT(tnt_lex(&l, &tk) == TNT_TK_UPDATE);
@@ -916,7 +932,7 @@ static void tt_tnt_lex_kw(struct tt_test *test) {
 static void tt_tnt_lex_stack(struct tt_test *test) {
 	unsigned char sz[] = "  1 'hey' ,.55";
 	struct tnt_lex l;
-	tnt_lex_init(&l, sz, sizeof(sz) - 1);
+	tnt_lex_init(&l, tnt_sql_keywords, sz, sizeof(sz) - 1);
 	struct tnt_tk *tk1, *tk2, *tk3, *tk4, *tk5, *tk6;
 	TT_ASSERT(tnt_lex(&l, &tk1) == TNT_TK_NUM32);
 	TT_ASSERT(tnt_lex(&l, &tk2) == TNT_TK_STRING);
@@ -942,7 +958,7 @@ static void tt_tnt_lex_stack(struct tt_test *test) {
 static void tt_tnt_lex_badstr1(struct tt_test *test) {
 	unsigned char sz[] = "  '";
 	struct tnt_lex l;
-	tnt_lex_init(&l, sz, sizeof(sz) - 1);
+	tnt_lex_init(&l, tnt_sql_keywords, sz, sizeof(sz) - 1);
 	struct tnt_tk *tk;
 	TT_ASSERT(tnt_lex(&l, &tk) == TNT_TK_ERROR);
 	tnt_lex_free(&l);
@@ -953,7 +969,7 @@ static void tt_tnt_lex_badstr1(struct tt_test *test) {
 static void tt_tnt_lex_badstr2(struct tt_test *test) {
 	unsigned char sz[] = "  '\n'";
 	struct tnt_lex l;
-	tnt_lex_init(&l, sz, sizeof(sz) - 1);
+	tnt_lex_init(&l, tnt_sql_keywords, sz, sizeof(sz) - 1);
 	struct tnt_tk *tk;
 	TT_ASSERT(tnt_lex(&l, &tk) == TNT_TK_ERROR);
 	tnt_lex_free(&l);
@@ -1134,6 +1150,7 @@ main(int argc, char * argv[])
 	tt_test(&t, "iterator tuple", tt_tnt_iter1);
 	tt_test(&t, "iterator tuple (single field)", tt_tnt_iter11);
 	tt_test(&t, "iterator tuple (tnt_field)", tt_tnt_iter2);
+	tt_test(&t, "iterator tuple (empty)", tt_tnt_iter4);
 	tt_test(&t, "iterator list", tt_tnt_iter3);
 	/* marshaling */
 	tt_test(&t, "marshaling ping", tt_tnt_marshal_ping);
